# coding: utf-8

import os
import shutil
from platform import machine, platform

import PyQt5.sip as sip
from com import *
from fastpip import PyEnv
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from settings import *
from ui import *
from utils import *
from utils.cip import ImportInspector
from utils.main import open_explorer
from utils.pyi import PyiTool
from utils.venv import VirtualEnv
from utils.widgets import LineEdit, TextEdit

from .messagebox import MessageBox


class PyinstallerToolWindow(Ui_pyinstaller_tool, QMainWindow):
    signal_update_pyinfo = pyqtSignal(str)
    signal_update_pyiinfo = pyqtSignal(str)
    signal_update_pyipbtext = pyqtSignal(str)
    COMBOBOX_DEFITEM = "当前打包配置"
    PYIVER_FMT = "Pyinstaller - {}"
    QREV_FNAME = QRegExpValidator(QRegExp(r'[^\\/:*?"<>|]*'))
    QREV_NUMBER = QRegExpValidator(QRegExp(r"[0-9]*"))

    def __init__(self, parent):
        super().__init__(parent)
        self.setupUi(self)
        self.widget_group = (
            self.tab_project_files,
            self.tab_build_control,
            self.tab_file_ver_info,
            self.tab_advanced_setup,
            self.pb_select_py_env,
            self.pb_reinstall_pyi,
            self.cb_log_level,
            self.le_output_name,
            self.pb_check_imports,
            self.pb_gen_executable,
            self.cb_prioritize_venv,
            self.uiPushButton_apply_config,
            self.uiPushButton_delete_config,
            self.uiPushButton_save_config,
        )
        self.le_vers_group = (
            self.le_file_version_0,
            self.le_file_version_1,
            self.le_file_version_2,
            self.le_file_version_3,
            self.le_product_version_0,
            self.le_product_version_1,
            self.le_product_version_2,
            self.le_product_version_3,
        )
        self.config = PyinstallerToolConfig()
        self.__setup_other_widgets()
        self.set_platform_info()
        self.thread_repo = ThreadRepo(500)
        self.toolwin_venv = None
        self.toolwin_pyenv = None
        self.pyi_tool = PyiTool()
        self.__envch_win = EnvironChosenWindow(self, self.__call_env_back)
        self.__impcheck_win = ImportsCheckWindow(self, self.__install_missings)
        self.signal_slot_connection()
        self.pyi_running_mov = QMovie(":/loading.gif")
        self.pyi_running_mov.setScaledSize(QSize(16, 16))
        self.__venv_creating_result = 1

    def __save_window_size(self):
        if self.isMaximized() or self.isMinimized():
            return
        self.config.window_size = self.width(), self.height()

    def closeEvent(self, event: QCloseEvent):
        if not self.thread_repo.is_empty():
            MessageBox(
                "提醒",
                "任务正在运行中，关闭此窗口后任务将在后台运行。\n请勿对相关目录进行任\
何操作，否则可能会造成打包失败！",
                QMessageBox.Warning,
            ).exec_()
        self.__save_window_size()
        self.config_widgets_to_cfg()
        self.config.save_config()

    def display(self):
        self.resize(*self.config.window_size)
        if self.isMaximized():
            self.showMaximized()
        else:
            self.showNormal()
        if not self.thread_repo.is_empty():
            return
        self.config_cfg_to_widgets(None)

    def __setup_other_widgets(self):
        # 替换“程序启动入口”LineEdit控件
        self.le_program_entry = LineEdit(Accept.File, {".py", ".pyc", ".pyw", ".spec"})
        self.le_program_entry.setToolTip(
            "要打包的程序的启动入口(*.py *.pyw *.pyc *.spec)，此项必填。\n"
            "如果指定了 SPEC 文件，则以下绝大部分项目文件及生成控制都将不生效。\n"
            "可将格式正确的文件拖放到此处。"
        )
        self.horizontalLayout_3.replaceWidget(
            self.le_program_entry_old, self.le_program_entry
        )
        self.le_program_entry_old.deleteLater()
        # 替换“其他模块搜索路径”TextEdit控件
        self.te_module_search_path = TextEdit(Accept.Dir)
        self.te_module_search_path.setToolTip(
            "对应选项：-p, --paths\n程序的其他模块的搜索路径(模块的父目录)，此项可留空。\
\n仅当 Pyinstaller 无法自动找到模块时使用，支持将文件夹直接拖放到此处。"
        )
        self.verticalLayout_3.replaceWidget(
            self.te_module_search_path_old, self.te_module_search_path
        )
        self.te_module_search_path_old.deleteLater()
        # 替换“非源代码资源文件”LineEdit控件
        self.te_other_data = TextEdit(Accept.File)
        self.te_other_data.setToolTip(
            """对应选项：--add-data\n非源代码性质的其他资源文件，例如一些图片、配置文件等，此项可留空。\n"""
            """注意：资源文件需是打包前程序真正使用的资源且在源代码根目录范围内，否则打包后程序可能无法运行。可将文件\
或者文件夹直接拖到此处。"""
        )
        self.verticalLayout_4.replaceWidget(self.te_other_data_old, self.te_other_data)
        self.te_other_data_old.deleteLater()
        # 替换“文件图标路径”LineEdit控件
        self.le_file_icon_path = LineEdit(Accept.File, {".ico", ".icns"})
        self.le_file_icon_path.setToolTip(
            "对应选项：-i, --icon\n生成的 exe 可执行文件使用的图标，支持 .ico 等图标文件。\n可将格式正确的文件拖放到此处。"
        )
        self.horizontalLayout_11.replaceWidget(
            self.le_file_icon_path_old, self.le_file_icon_path
        )
        self.le_file_icon_path_old.deleteLater()
        if sip.isdeleted(self.QREV_NUMBER) or sip.isdeleted(self.QREV_FNAME):
            PyinstallerToolWindow.rebuild_validators()
        for line_edit in self.le_vers_group:
            line_edit.setValidator(self.QREV_NUMBER)
        self.le_output_name.setValidator(self.QREV_FNAME)
        self.le_runtime_tmpdir.setValidator(self.QREV_FNAME)
        self.splitter.setSizes((10000, 10000))

    @classmethod
    def rebuild_validators(cls):
        """
        用于解决以编程方式第二次执行 run_pykit 函数出现 x deleted 的问题
        """
        cls.QREV_NUMBER = QRegExpValidator(QRegExp(r"[0-9]*"))
        cls.QREV_FNAME = QRegExpValidator(QRegExp(r'[^\\/:*?"<>|]*'))

    def signal_slot_connection(self):
        self.pyi_tool.completed.connect(self.after_task_completed)
        self.pyi_tool.stdout.connect(self.te_pyi_out_stream.append)
        self.pb_select_py_env.clicked.connect(self.__envch_win.initialize)
        self.le_program_entry.textChanged.connect(self.set_le_project_root)
        self.pb_select_module_search_path.clicked.connect(
            self.set_te_module_search_path
        )
        self.pb_select_program_entry.clicked.connect(self.set_le_program_entry)
        self.pb_up_level_root.clicked.connect(lambda: self.project_root_level(1))
        self.pb_reset_root_level.clicked.connect(lambda: self.project_root_level(0))
        self.pb_clear_module_search_path.clicked.connect(
            self.te_module_search_path.clear
        )
        self.pb_select_other_data.clicked.connect(self.set_te_other_data)
        self.pb_clear_other_data.clicked.connect(self.te_other_data.clear)
        self.pb_select_file_icon.clicked.connect(self.set_le_file_icon_path)
        self.pb_select_spec_dir.clicked.connect(self.set_le_spec_dir)
        self.pb_select_temp_working_dir.clicked.connect(self.set_le_temp_working_dir)
        self.pb_select_output_dir.clicked.connect(self.set_le_output_dir)
        self.pb_select_upx_search_path.clicked.connect(self.set_le_upx_search_path)
        self.pb_gen_executable.clicked.connect(self.build_executable)
        self.pb_reinstall_pyi.clicked.connect(self.reinstall_pyinstaller)
        self.pb_check_imports.clicked.connect(self.check_project_imports)
        self.pb_clear_hidden_imports.clicked.connect(self.pte_hidden_imports.clear)
        self.pb_clear_exclude_module.clicked.connect(self.pte_exclude_modules.clear)
        self.uiPushButton_save_config.clicked.connect(self.store_current_config)
        self.uiPushButton_apply_config.clicked.connect(self.apply_selected_config)
        self.uiPushButton_delete_config.clicked.connect(self.delete_selected_config)
        self.signal_update_pyinfo.connect(self.lb_py_info.setText)
        self.signal_update_pyiinfo.connect(self.lb_pyi_info.setText)
        self.signal_update_pyipbtext.connect(self.pb_reinstall_pyi.setText)

    def check_project_imports(self):
        self.config_widgets_to_cfg()
        self.toolwin_venv = VirtualEnv(self.config.curconfig.project_root)
        self.toolwin_venv.find_project_venv()
        if self.config.curconfig.prioritize_venv:
            if not self.toolwin_venv.venv_exists:
                return MessageBox(
                    "提示",
                    "项目目录下不存在虚拟环境，请直接点击生成可执行文件，程序会引导创建虚拟环境。",
                    QMessageBox.Warning,
                ).exec_()
            environ = self.toolwin_venv
        else:
            environ = self.toolwin_pyenv
        if not environ:
            MessageBox(
                "提示",
                "还没有选择主 Python 环境或者勾选优先使用项目目录下的虚拟环境。",
                QMessageBox.Warning,
            ).exec_()
            return
        project_root = self.config.curconfig.project_root
        if not project_root:
            MessageBox(
                "提示",
                "源代码根目录未填写！",
                QMessageBox.Warning,
            ).exec_()
            return
        if not os.path.isdir(project_root):
            MessageBox(
                "提示",
                "源代码根目录不存在！",
                QMessageBox.Warning,
            ).exec_()
            return
        dist_dir = self.config.curconfig.distribution_dir
        if not dist_dir:
            dist_dir = os.path.join(project_root, "dist")
        elif not os.path.isabs(dist_dir):
            dist_dir = os.path.join(project_root, dist_dir)
        missings = list()
        self.pyi_tool.initialize(environ.env_path, project_root)
        if not self.pyi_tool.pyi_ready:
            missings.append(("打包功能核心模块", {}, {"pyinstaller"}))

        def get_missing_imps():
            inspector = ImportInspector(
                environ.env_path,
                project_root,
                [self.toolwin_venv.env_path, dist_dir],
            )
            if (
                self.config.curconfig.encryption_key
                and "tinyaes" not in inspector.importables
            ):
                missings.append(("字节码加密功能", {}, {"tinyaes"}))
            missings.extend(inspector.get_missing_items())

        thread_check_imp = QThreadModel(get_missing_imps)
        thread_check_imp.at_start(
            self.__lock_widgets,
            lambda: self.__show_running("正在分析项目依赖在环境中的安装情况..."),
        )
        thread_check_imp.at_finish(
            self.__hide_running,
            self.__release_widgets,
            lambda: self.__impcheck_win.display_result(environ, missings),
        )
        thread_check_imp.start()
        self.thread_repo.put(thread_check_imp, 1)

    def set_le_program_entry(self):
        selected_file = self.__ask_file_or_dir_path(
            "选择程序启动入口",
            self.config.curconfig.project_root,
            ext_filter="脚本文件 (*.py *.pyc *.pyw *.spec)",
        )[0]
        if not selected_file:
            return
        self.le_program_entry.setText(selected_file)

    def set_le_project_root(self):
        root = os.path.dirname(self.le_program_entry.text())
        self.le_project_root.setText(root)
        self.config.curconfig.project_root = root

    def set_te_module_search_path(self):
        selected_dir = self.__ask_file_or_dir_path(
            "其他模块搜索目录", self.config.curconfig.project_root, ch=Accept.Dir
        )[0]
        if not selected_dir:
            return
        self.te_module_search_path.append(selected_dir)

    def set_te_other_data(self):
        selected_files = self.__ask_file_or_dir_path(
            "选择非源码资源文件", self.config.curconfig.project_root, multi=True
        )
        if not selected_files:
            return
        self.te_other_data.append("\n".join(selected_files))

    def set_le_file_icon_path(self):
        selected_file = self.__ask_file_or_dir_path(
            "选择可执行文件图标",
            self.config.curconfig.project_root,
            ext_filter="图标文件 (*.ico *.icns)",
        )[0]
        if not selected_file:
            return
        self.le_file_icon_path.setText(selected_file)

    def set_le_spec_dir(self):
        selected_dir = self.__ask_file_or_dir_path(
            "选择SPEC文件储存目录", self.config.curconfig.project_root, ch=Accept.Dir
        )[0]
        if not selected_dir:
            return
        self.le_spec_dir.setText(selected_dir)

    def set_le_temp_working_dir(self):
        selected_dir = self.__ask_file_or_dir_path(
            "选择临时文件目录", self.config.curconfig.project_root, ch=Accept.Dir
        )[0]
        if not selected_dir:
            return
        self.le_temp_working_dir.setText(selected_dir)

    def set_le_output_dir(self):
        selected_dir = self.__ask_file_or_dir_path(
            "选择生成文件储存目录", self.config.curconfig.project_root, ch=Accept.Dir
        )[0]
        if not selected_dir:
            return
        self.le_output_dir.setText(selected_dir)

    def set_le_upx_search_path(self):
        selected_dir = self.__ask_file_or_dir_path(
            "选择UPX程序搜索目录", self.config.curconfig.project_root, ch=Accept.Dir
        )[0]
        if not selected_dir:
            return
        self.le_upx_search_path.setText(selected_dir)

    def __ask_file_or_dir_path(
        self, title="", start="", ch=Accept.File, multi=False, ext_filter="所有文件 (*)"
    ):
        file_dir_paths = []
        if ch == Accept.File and multi:
            if not title:
                title = "选择多文件"
            path_getter = QFileDialog.getOpenFileNames
        elif ch == Accept.File and not multi:
            if not title:
                title = "选择文件"
            path_getter = QFileDialog.getOpenFileName
        elif ch == Accept.Dir:
            if not title:
                title = "选择文件夹"
            path_getter = QFileDialog.getExistingDirectory
        else:
            return file_dir_paths
        if ch == Accept.File and not multi:
            path = path_getter(self, title, start, ext_filter)[0]
            if not path:
                file_dir_paths.append("")
            else:
                file_dir_paths.append(os.path.realpath(path))
        elif ch == Accept.File and multi:
            paths = path_getter(self, title, start, ext_filter)[0]
            file_dir_paths.extend(os.path.realpath(path) for path in paths if path)
            if not file_dir_paths:
                file_dir_paths.append("")
        elif ch == Accept.Dir:
            path = path_getter(self, title, start)
            if not path:
                file_dir_paths.append("")
            else:
                file_dir_paths.append(os.path.realpath(path))
        return file_dir_paths

    def __call_env_back(self, env):
        self.toolwin_pyenv = env
        self.pyi_tool.initialize(
            self.toolwin_pyenv.env_path,
            self.config.curconfig.project_root,
        )
        self.load_version_information_lazily(refresh=False)

    def config_cfg_to_widgets(self, cfg_name):
        if cfg_name is not None:
            self.config.checkout_cfg(cfg_name)
        self.update_configure_combobox_items()
        self.load_version_information_lazily(refresh=True)
        self.le_program_entry.setText(self.config.curconfig.script_path)
        self.le_project_root.setText(self.config.curconfig.project_root)
        self.te_module_search_path.setText(
            "\n".join(self.config.curconfig.module_paths)
        )
        self.te_other_data.setText(
            "\n".join(pg[0] for pg in self.config.curconfig.other_datas)
        )
        self.le_file_icon_path.setText(self.config.curconfig.icon_path)
        if self.config.curconfig.onedir_bundle:
            self.rb_pack_to_one_dir.setChecked(True)
        else:
            self.rb_pack_to_one_file.setChecked(True)
        self.cb_execute_with_console.setChecked(
            self.config.curconfig.provide_console
        )
        self.cb_without_confirm.setChecked(self.config.curconfig.no_confirm)
        self.cb_use_upx.setChecked(not self.config.curconfig.donot_use_upx)
        self.cb_clean_before_build.setChecked(self.config.curconfig.clean_building)
        self.cb_write_info_to_exec.setChecked(self.config.curconfig.add_verfile)
        self.le_temp_working_dir.setText(self.config.curconfig.working_dir)
        self.le_output_dir.setText(self.config.curconfig.distribution_dir)
        self.le_spec_dir.setText(self.config.curconfig.spec_dir)
        self.le_upx_search_path.setText(self.config.curconfig.upx_dir)
        self.te_upx_exclude_files.setText(
            "\n".join(self.config.curconfig.upx_excludes)
        )
        self.le_output_name.setText(self.config.curconfig.bundle_spec_name)
        self.cb_log_level.setCurrentText(self.config.curconfig.log_level)
        self.set_file_ver_info_text()
        self.set_pyi_debug_options()
        self.le_runtime_tmpdir.setText(self.config.curconfig.runtime_tmpdir)
        self.cb_prioritize_venv.setChecked(self.config.curconfig.prioritize_venv)
        self.le_bytecode_encryption_key.setText(self.config.curconfig.encryption_key)
        self.cb_explorer_show.setChecked(self.config.curconfig.open_dist_folder)
        self.cb_delete_working_dir.setChecked(
            self.config.curconfig.delete_working_dir
        )
        self.cb_delete_spec_file.setChecked(self.config.curconfig.delete_spec_file)
        self.pte_hidden_imports.setPlainText(
            "\n".join(self.config.curconfig.hidden_imports)
        )
        self.pte_exclude_modules.setPlainText(
            "\n".join(self.config.curconfig.exclude_modules)
        )
        self.cb_uac_admin.setChecked(self.config.curconfig.uac_admin)

    def config_widgets_to_cfg(self):
        self.config.curconfig.script_path = self.le_program_entry.local_path
        self.config.curconfig.bundle_spec_name = self.le_output_name.text()
        project_root = self.le_project_root.text()
        self.config.curconfig.project_root = project_root
        self.config.curconfig.module_paths = self.te_module_search_path.local_paths
        self.config.curconfig.other_datas = self.__gen_abs_rel_groups(project_root)
        self.config.curconfig.icon_path = self.le_file_icon_path.local_path
        self.config.curconfig.onedir_bundle = self.rb_pack_to_one_dir.isChecked()
        self.config.curconfig.provide_console = (
            self.cb_execute_with_console.isChecked()
        )
        self.config.curconfig.no_confirm = self.cb_without_confirm.isChecked()
        self.config.curconfig.donot_use_upx = not self.cb_use_upx.isChecked()
        self.config.curconfig.clean_building = self.cb_clean_before_build.isChecked()
        self.config.curconfig.add_verfile = self.cb_write_info_to_exec.isChecked()
        self.config.curconfig.working_dir = self.le_temp_working_dir.text()
        self.config.curconfig.distribution_dir = self.le_output_dir.text()
        self.config.curconfig.spec_dir = self.le_spec_dir.text()
        self.config.curconfig.upx_dir = self.le_upx_search_path.text()
        self.config.curconfig.upx_excludes = [
            s for s in self.te_upx_exclude_files.toPlainText().split("\n") if s
        ]
        if self.toolwin_pyenv is not None:
            self.config.curconfig.environ_path = self.toolwin_pyenv.env_path
        self.config.curconfig.log_level = self.cb_log_level.currentText()
        self.config.curconfig.version_info = self.file_ver_info_text()
        self.config.curconfig.debug_options = self.get_pyi_debug_options()
        self.config.curconfig.runtime_tmpdir = self.le_runtime_tmpdir.text()
        self.config.curconfig.prioritize_venv = self.cb_prioritize_venv.isChecked()
        self.config.curconfig.encryption_key = self.le_bytecode_encryption_key.text()
        self.config.curconfig.open_dist_folder = self.cb_explorer_show.isChecked()
        self.config.curconfig.delete_working_dir = (
            self.cb_delete_working_dir.isChecked()
        )
        self.config.curconfig.delete_spec_file = self.cb_delete_spec_file.isChecked()
        self.config.curconfig.uac_admin = self.cb_uac_admin.isChecked()
        self.config.curconfig.hidden_imports = [
            s for s in self.pte_hidden_imports.toPlainText().split("\n") if s
        ]
        self.config.curconfig.exclude_modules = [
            s for s in self.pte_exclude_modules.toPlainText().split("\n") if s
        ]

    def __gen_abs_rel_groups(self, starting_point):
        """获取其他要打包的文件的本地路径和与源代码根目录的相对位置。"""
        other_data_local_paths = self.te_other_data.local_paths
        abs_rel_path_groups = []
        for abs_path in other_data_local_paths:
            try:
                rel_path = os.path.relpath(os.path.dirname(abs_path), starting_point)
            except Exception:
                continue
            abs_rel_path_groups.append((abs_path, rel_path))
        return abs_rel_path_groups

    def get_pyi_debug_options(self):
        return {
            "imports": self.cb_db_imports.isChecked(),
            "bootloader": self.cb_db_bootloader.isChecked(),
            "noarchive": self.cb_db_noarchive.isChecked(),
        }

    def set_pyi_debug_options(self):
        dbo = self.config.curconfig.debug_options
        self.cb_db_imports.setChecked(dbo.get("imports", False))
        self.cb_db_bootloader.setChecked(dbo.get("bootloader", False))
        self.cb_db_noarchive.setChecked(dbo.get("noarchive", False))

    def file_ver_info_text(self):
        file_vers = tuple(int(x.text() or 0) for x in self.le_vers_group[:4])
        prod_vers = tuple(int(x.text() or 0) for x in self.le_vers_group[4:])
        return {
            "$filevers$": str(file_vers),
            "$prodvers$": str(prod_vers),
            "$CompanyName$": self.le_company_name.text(),
            "$FileDescription$": self.le_file_description.text(),
            "$FileVersion$": ".".join(map(str, file_vers)),
            "$LegalCopyright$": self.le_legal_copyright.text(),
            "$OriginalFilename$": self.le_original_filename.text(),
            "$ProductName$": self.le_product_name.text(),
            "$ProductVersion$": ".".join(map(str, prod_vers)),
            "$LegalTrademarks$": self.le_legal_trademarks.text(),
        }

    def set_file_ver_info_text(self):
        version_info = self.config.curconfig.version_info
        self.le_file_description.setText(version_info.get("$FileDescription$", ""))
        self.le_company_name.setText(version_info.get("$CompanyName$", ""))
        for ind, val in enumerate(
            version_info.get("$FileVersion$", "0.0.0.0").split(".")
        ):
            self.le_vers_group[ind].setText(val)
        self.le_product_name.setText(version_info.get("$ProductName$", ""))
        for ind, val in enumerate(
            version_info.get("$ProductVersion$", "0.0.0.0").split(".")
        ):
            self.le_vers_group[ind + 4].setText(val)
        self.le_legal_copyright.setText(version_info.get("$LegalCopyright$", ""))
        self.le_legal_trademarks.setText(version_info.get("$LegalTrademarks$", ""))
        self.le_original_filename.setText(version_info.get("$OriginalFilename$", ""))

    def reinstall_pyinstaller(self):
        if not self.toolwin_pyenv:
            return MessageBox(
                "提示",
                "当前未选择任何 Python 环境。",
                QMessageBox.Warning,
            ).exec_()
        # NewMessageBox的exec_方法返回0才是选择"确定"按钮
        if MessageBox(
            "安装",
            "确定安装 Pyinstaller 吗？",
            QMessageBox.Question,
            (("accept", "确定"), ("reject", "取消")),
        ).exec_():
            return

        def do_reinstall_pyi():
            self.toolwin_pyenv.uninstall("pyinstaller")
            self.toolwin_pyenv.install("pyinstaller", upgrade=1)

        thread_reinstall = QThreadModel(target=do_reinstall_pyi)
        thread_reinstall.at_start(
            self.__lock_widgets,
            lambda: self.__show_running("正在安装 Pyinstaller..."),
        )
        thread_reinstall.at_finish(
            self.__hide_running,
            self.__release_widgets,
            lambda: self.load_version_information_lazily(False),
        )
        thread_reinstall.start()
        self.thread_repo.put(thread_reinstall, 0)

    def set_platform_info(self):
        self.lb_platform_info.setText(f"{platform()}-{machine()}")

    def project_root_level(self, option):
        """
        option 为 0 表示重置路径，1 表示设为上一级目录
        """
        root = self.le_project_root.text()
        if not root:
            return
        if option == 1:
            _path = os.path.dirname(root)
        elif option == 0:
            deep = self.le_program_entry.text()
            if not deep:
                return
            _path = os.path.dirname(deep)
        else:
            raise Exception("选项错误")
        self.le_project_root.setText(_path)

    def __check_requireds(self):
        self.config_widgets_to_cfg()
        program_entry = self.config.curconfig.script_path
        if not program_entry:
            MessageBox(
                "错误",
                "程序启动入口未填写！",
                QMessageBox.Critical,
            ).exec_()
            return False
        if not os.path.isfile(program_entry):
            MessageBox(
                "错误",
                "程序启动入口文件不存在！",
                QMessageBox.Critical,
            ).exec_()
            return False
        icon_path = self.config.curconfig.icon_path
        if icon_path != "" and not os.path.isfile(icon_path):
            MessageBox(
                "错误",
                "程序图标文件不存在！",
                QMessageBox.Critical,
            ).exec_()
            return False
        return True

    def __show_running(self, msg):
        self.lb_running_tip.setText(msg)
        self.lb_running_gif.setMovie(self.pyi_running_mov)
        self.pyi_running_mov.start()

    def __hide_running(self):
        self.pyi_running_mov.stop()
        self.lb_running_gif.clear()
        self.lb_running_tip.clear()

    def __lock_widgets(self):
        for widget in self.widget_group:
            widget.setEnabled(False)

    def __release_widgets(self):
        for widget in self.widget_group:
            widget.setEnabled(True)

    def importance_operation_start(self, string):
        def function():
            self.__lock_widgets()
            self.__show_running(string)

        return function

    def importance_operation_finish(self):
        self.__hide_running()
        self.__release_widgets()

    def __get_program_name(self):
        program_name = self.config.curconfig.bundle_spec_name
        if not program_name:
            program_name = os.path.splitext(
                os.path.basename(self.config.curconfig.script_path)
            )[0]
        return program_name

    def open_explorer_select_file(self):
        program_name = self.__get_program_name()
        if self.rb_pack_to_one_file.isChecked():
            sub_directory = ""
        else:
            sub_directory = program_name
        final_execfn, ext = os.path.splitext(program_name)
        if ext.lower() != ".exe":
            final_execfn = program_name
        dist_folder = self.config.curconfig.distribution_dir
        if not dist_folder:
            dist_folder = os.path.join(self.config.curconfig.project_root, "dist")
        elif not os.path.isabs(dist_folder):
            dist_folder = os.path.join(
                self.config.curconfig.project_root, dist_folder
            )
        explorer_selected = (
            os.path.join(dist_folder, sub_directory, final_execfn) + ".exe"
        )
        open_explorer(explorer_selected, "select")

    def delete_spec_file(self):
        spec_file_dir = self.config.curconfig.spec_dir
        if not spec_file_dir:
            spec_file_dir = self.config.curconfig.project_root
        elif not os.path.isabs(spec_file_dir):
            spec_file_dir = os.path.join(
                self.config.curconfig.project_root, spec_file_dir
            )
        program_name = self.__get_program_name()
        spec_file_path = os.path.join(spec_file_dir, program_name) + ".spec"
        try:
            os.remove(spec_file_path)
        except Exception:
            pass
        # 自定义 spec 文件储存目录的情况下，考虑到目录内可能有其他文件
        # 所以只考虑删除 spec 文件，不删除自定义目录，以防误删其他无关文件

    def delete_working_dir(self):
        program_name = self.__get_program_name()
        custom_working_dir = self.config.curconfig.working_dir
        if not custom_working_dir:
            working_dir_root = os.path.join(
                self.config.curconfig.project_root, "build"
            )
        elif not os.path.isabs(custom_working_dir):
            working_dir_root = os.path.join(
                self.config.curconfig.project_root, custom_working_dir
            )
        else:
            working_dir_root = os.path.join(custom_working_dir, program_name)
        thread_delete_working = QThreadModel(shutil.rmtree, (working_dir_root, True))
        thread_delete_working.start()
        self.thread_repo.put(thread_delete_working)

    def after_task_completed(self, retcode):
        if retcode == 0:
            if self.cb_explorer_show.isChecked():
                self.open_explorer_select_file()
            if self.cb_delete_spec_file.isChecked():
                self.delete_spec_file()
            if self.cb_delete_working_dir.isChecked():
                self.delete_working_dir()
            MessageBox(
                "任务结束",
                "Python 程序已打包完成！",
            ).exec_()
        else:
            MessageBox(
                "任务结束",
                "打包失败，请检查错误信息！",
                QMessageBox.Critical,
            ).exec_()

    def creating_virtualenv(self):
        dist_dir = self.config.curconfig.distribution_dir
        if not dist_dir:
            dist_dir = os.path.join(self.toolwin_venv.project, "dist")
        elif not os.path.isabs(dist_dir):
            dist_dir = os.path.join(self.toolwin_venv.project, dist_dir)
        if self.toolwin_venv.create_project_venv(self.toolwin_pyenv.interpreter):
            import_inspect = ImportInspector(
                self.toolwin_venv.env_path,
                self.toolwin_venv.project,
                [self.toolwin_venv.env_path, dist_dir],
            )
            missings = set()
            for i in import_inspect.get_missing_items():
                missings.update(i[2])
            missings.add("pyinstaller")
            if self.config.curconfig.encryption_key:
                missings.add("tinyaes")
            for pkg in missings:
                self.toolwin_venv.install(pkg)
            self.__venv_creating_result = 0  # 虚拟环境创建成功
        else:
            self.__venv_creating_result = 1  # 虚拟环境创建不成功

    def build_executable(self):
        if not self.__check_requireds():
            return
        if self.uiComboBox_saved_config.currentIndex() != 0:
            result = MessageBox(
                "提示",
                "你选择的打包配置似乎还没有应用以使其生效，确定开始打包吗？",
                QMessageBox.Warning,
                (("accept", "确定"), ("reject", "取消")),
            ).exec_()
            if result != 0:
                return
        self.te_pyi_out_stream.clear()
        if self.config.curconfig.prioritize_venv:
            self.toolwin_venv = VirtualEnv(self.config.curconfig.project_root)
            self.__venv_creating_result = 0
            if not self.toolwin_venv.find_project_venv():
                role = MessageBox(
                    "提示",
                    "项目目录下不存在虚拟环境，请选择合适选项。",
                    QMessageBox.Warning,
                    (
                        ("accept", "使用主环境"),
                        ("destructive", "创建虚拟环境"),
                        ("reject", "取消"),
                    ),
                ).exec_()
                if role == 0:
                    using_py_path = self.config.curconfig.environ_path
                elif role == 1:
                    if self.toolwin_pyenv is None or not self.toolwin_pyenv.env_path:
                        return MessageBox(
                            "提示",
                            "没有选择主 Python 环境或者主 Python 环境不可用。",
                            QMessageBox.Warning,
                        ).exec_()
                    thread_venv_creating = QThreadModel(self.creating_virtualenv)
                    thread_venv_creating.at_start(
                        self.__lock_widgets,
                        lambda: self.__show_running("正在创建虚拟环境并安装模块..."),
                    )
                    thread_venv_creating.at_finish(
                        self.__hide_running,
                        self.__release_widgets,
                        self.build_executable,
                    )
                    thread_venv_creating.start()
                    self.thread_repo.put(thread_venv_creating, 0)
                    return
                else:
                    return
            else:
                if self.__venv_creating_result != 0:
                    return MessageBox(
                        "错误", "项目目录下的虚拟环境创建失败。", QMessageBox.Critical
                    ).exec_()
                using_py_path = self.toolwin_venv.env_path
        else:
            using_py_path = self.config.curconfig.environ_path
        self.pyi_tool.initialize(
            using_py_path,
            self.config.curconfig.project_root,
        )
        if not self.pyi_tool.pyi_ready:
            MessageBox(
                "Pyinstaller 不可用",
                "请点击右上角'选择环境'按钮选择打包环境，再点击'安装'按钮将 Pyinstaller 安装到所选的打包环境。\n"
                "如果勾选了'使用项目目录下的虚拟环境而不是以上环境'，请点击'环境检查'按钮检查环境中缺失的模块并'一键安装'。",
                QMessageBox.Warning,
            ).exec_()
            return
        self.pyi_tool.prepare_cmds(self.config.curconfig)
        thread_build = QThreadModel(self.pyi_tool.execute_cmd)
        thread_build.at_start(
            self.__lock_widgets,
            lambda: self.__show_running("正在生成可执行文件..."),
        )
        thread_build.at_finish(self.__hide_running, self.__release_widgets)
        thread_build.start()
        self.thread_repo.put(thread_build, 0)

    def __install_missings(self, missings):
        if not missings:
            MessageBox(
                "提示",
                "没有缺失的模块，无需安装。",
            ).exec_()
            return self.__impcheck_win.close()
        if (
            MessageBox(
                "安装",
                "确定将所有缺失模块安装至所选 Python 环境中吗？",
                QMessageBox.Question,
                (("accept", "确定"), ("reject", "取消")),
            ).exec_()
            != 0
        ):
            return
        if self.config.curconfig.prioritize_venv:
            environ = self.toolwin_venv
        else:
            environ = self.toolwin_pyenv

        def install_pkgs():
            names_for_install = set()
            for imp in missings:
                if imp not in import_name:
                    names_for_install.add(imp)
                else:
                    names_for_install.add(import_name[imp])
            for imp in names_for_install:
                environ.install(imp)

        self.__impcheck_win.close()
        thread_install_missings = QThreadModel(install_pkgs)
        thread_install_missings.at_start(
            self.__lock_widgets,
            lambda: self.__show_running("正在安装缺失模块..."),
        )
        thread_install_missings.at_finish(
            self.__hide_running,
            self.__release_widgets,
            lambda: MessageBox(
                "完成",
                "已完成安装流程，请重新检查是否安装成功。",
                QMessageBox.Information,
            ).exec_(),
        )
        thread_install_missings.start()
        self.thread_repo.put(thread_install_missings, 0)

    def update_configure_combobox_items(self):
        self.uiComboBox_saved_config.clear()
        self.uiComboBox_saved_config.addItem(self.COMBOBOX_DEFITEM)
        self.uiComboBox_saved_config.addItems(self.config.multicfg)

    def store_current_config(self):
        text = self.uiLineEdit_config_remark.text()
        if not text:
            return MessageBox(
                "提示",
                "还没有输入备注名称。",
            ).exec_()
        if text == self.COMBOBOX_DEFITEM:
            return MessageBox(
                "提示",
                f"不能用“{text}”作为备注名。",
            ).exec_()
        if text in self.config.multicfg:
            result = MessageBox(
                "提示",
                f"当前配置列表已存在名为“{text}”的配置，是否覆盖？",
                QMessageBox.Warning,
                (("accept", "确定"), ("reject", "取消")),
            ).exec_()
            if result != 0:
                return
        self.uiLineEdit_config_remark.clear()
        self.config_widgets_to_cfg()
        self.config.store_curcfg(text)
        self.update_configure_combobox_items()
        MessageBox("提示", f"配置已以此备注名保存：{text}。").exec_()

    def delete_selected_config(self):
        if not len(self.config.multicfg):
            return MessageBox("提示", "没有已保存的配置。").exec_()
        text = self.uiComboBox_saved_config.currentText()
        if not text:
            return MessageBox("提示", "没有选择任何配置。").exec_()
        if text == self.COMBOBOX_DEFITEM:
            return
        if text not in self.config.multicfg:
            return
        if (
            MessageBox(
                "提示",
                f"确认删除当前选中的配置？",
                QMessageBox.Warning,
                (("accept", "确定"), ("reject", "取消")),
            ).exec_()
            != 0
        ):
            return
        del self.config.multicfg[text]
        self.update_configure_combobox_items()

    def apply_selected_config(self):
        if not len(self.config.multicfg):
            return MessageBox("提示", "没有已保存的配置。").exec_()
        text = self.uiComboBox_saved_config.currentText()
        if not text:
            return MessageBox("提示", "没有选择任何配置。").exec_()
        if text == self.COMBOBOX_DEFITEM:
            return
        self.config_cfg_to_widgets(text)
        self.uiComboBox_saved_config.setCurrentText(self.COMBOBOX_DEFITEM)

    def load_version_information_lazily(self, refresh):
        """为显示 Python 和 Pyinstaller 版本信息这两个耗时操作提供延迟加载的方法"""

        def do_load_version_information():
            if refresh:
                python_path = self.config.curconfig.environ_path
                if python_path:
                    self.toolwin_pyenv = PyEnv(python_path)
                    self.pyi_tool.initialize(
                        python_path, self.config.curconfig.project_root
                    )
            if self.toolwin_pyenv is None:
                pyinfo = ""
                pyiinfo = ""
                pbtext = "安装"
            else:
                pyiinfo = self.pyi_tool.pyi_info()
                pyinfo = self.toolwin_pyenv.py_info()
                if pyiinfo == "0.0":
                    pbtext = "安装"
                else:
                    pbtext = "重新安装"
                pyiinfo = self.PYIVER_FMT.format(pyiinfo)
            self.signal_update_pyinfo.emit(pyinfo)
            self.signal_update_pyiinfo.emit(pyiinfo)
            self.signal_update_pyipbtext.emit(pbtext)

        thread_load_info = QThreadModel(do_load_version_information)
        thread_load_info.at_start(self.importance_operation_start("正在加载配置..."))
        thread_load_info.at_finish(self.importance_operation_finish)
        thread_load_info.start()
        self.thread_repo.put(thread_load_info, 1)


class EnvironChosenWindow(Ui_environ_chosen, QMainWindow):
    def __init__(self, parent: PyinstallerToolWindow, callback):
        self.__parent = parent
        super().__init__(parent)
        self.setupUi(self)
        self.setWindowFlags(Qt.Window | Qt.WindowCloseButtonHint)
        self.__envlist = None
        self.__call_back = callback
        self.lw_environ_list.clicked.connect(self.__call_environ_back)

    def __env_list_update(self):
        self.lw_environ_list.clear()
        row_size = QSize(0, 28)
        for env in self.__envlist:
            item = QListWidgetItem(str(env))
            item.setSizeHint(row_size)
            self.lw_environ_list.addItem(item)

    def __save_window_size(self):
        if self.isMaximized() or self.isMinimized():
            return
        self.__parent.config.envch_winsize = self.width(), self.height()

    def closeEvent(self, event: QCloseEvent):
        self.__save_window_size()

    def __call_environ_back(self):
        self.close()
        selected = self.lw_environ_list.currentRow()
        if selected != -1:
            self.__call_back(self.__envlist[selected])

    def initialize(self):
        self.resize(*self.__parent.config.envch_winsize)
        self.showNormal()
        self.__envlist = [PyEnv(p) for p in self.__parent.config.pypaths]
        self.__env_list_update()


class ImportsCheckWindow(Ui_imports_check, QMainWindow):
    def __init__(self, parent: PyinstallerToolWindow, call_install):
        super().__init__(parent)
        self.__parent = parent
        self.setupUi(self)
        self.setWindowFlags(Qt.Window | Qt.WindowCloseButtonHint)
        self.__setup_other_widgets()
        self.pb_confirm.clicked.connect(self.close)
        self.pb_install_all_missing.clicked.connect(self.__call_install_back)
        self.__missing_modules = None
        self.__call_install = call_install

    def __setup_other_widgets(self):
        self.tw_missing_imports.setColumnWidth(0, 260)
        self.tw_missing_imports.setColumnWidth(1, 360)
        self.tw_missing_imports.horizontalHeader().setSectionResizeMode(
            QHeaderView.Interactive
        )
        self.tw_missing_imports.horizontalHeader().setSectionResizeMode(
            2, QHeaderView.Stretch
        )

    def __call_install_back(self):
        self.close()  # 触发 closeEvent
        self.__call_install(self.__missing_modules)

    def __save_window_size(self):
        if self.isMaximized() or self.isMinimized():
            return
        self.__parent.config.impcheck_winsize = self.width(), self.height()

    def closeEvent(self, event: QCloseEvent):
        self.__save_window_size()

    def display_window(self):
        self.resize(*self.__parent.config.impcheck_winsize)
        self.showNormal()

    def display_result(self, environ, missings_list):
        """
        missings_list: [(filepath, {imps...}, {missings...})...]
        """
        if environ:
            self.le_cip_cur_env.setText(str(environ))
        self.__missing_modules = set()
        for *_, m in missings_list:
            self.__missing_modules.update(m)
        self.tw_missing_imports.clearContents()
        self.tw_missing_imports.setRowCount(len(missings_list))
        for index, value in enumerate(missings_list):
            # value[0] 即 filepath 为 None，依
            # ImportInspector.missing_items 返回值特点可知没有可以打开的文件
            if value[0] is None:
                break
            self.tw_missing_imports.setVerticalHeaderItem(
                index, QTableWidgetItem(f" {index + 1} ")
            )
            item1 = QTableWidgetItem(os.path.basename(value[0]))
            item2 = QTableWidgetItem("，".join(value[1]))
            item3 = QTableWidgetItem("，".join(value[2]))
            item1.setToolTip(value[0])
            item2.setToolTip("\n".join(value[1]))
            item3.setToolTip("\n".join(value[2]))
            self.tw_missing_imports.setItem(index, 0, item1)
            self.tw_missing_imports.setItem(index, 1, item2)
            self.tw_missing_imports.setItem(index, 2, item3)
        self.display_window()



# coding: utf-8

################################################################################
# MIT License

# Copyright (c) 2020-2022 hrp/hrpzcf

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
################################################################################
# Formatted with black
################################################################################

import sys
from os import path

from fastpip import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *

sys.path.append(path.dirname(__file__))  # rpk.exe 入口点所需

from __info__ import *
from com import *
from logic import *
from res.res import *
from settings import *
from ui import *

if VERNUM[0] != REQ_FPVER[0]:
    raise Exception(f"当前环境的 fastpip 主版本号({VERNUM[0]})非本程序要求：{REQ_FPVER[0]}")
if VERNUM[1] < REQ_FPVER[1]:
    raise Exception(f"当前环境的 fastpip 次版本号({VERNUM[1]})低于本程序要求：{REQ_FPVER[1]}")
elif VERNUM[1] == REQ_FPVER[1] and VERNUM[2] < REQ_FPVER[2]:
    raise Exception(f"当前环境的 fastpip 修订号({VERNUM[2]})低于本程序要求：{REQ_FPVER[2]}")
################################################################
# 版本号的定义：主版本号.次版本号.修订号，对 fastpip 的版本号要求：
# 1. 主版本号必须与要求一致，次版本号必须大于等于要求的次版本号
# 3. 如次版本号等于要求的次版本号，则修订号必须大于等于要求的修订号
################################################################


class MainEntrance(Ui_main_entrance, QMainWindow):
    def __init__(self, config: MainEntranceConfig):
        super().__init__()
        self.setupUi(self)
        self.setWindowFlags(Qt.WindowMinimizeButtonHint | Qt.WindowCloseButtonHint)
        self.setWindowTitle(NAME)
        self.__config = config
        self.__pkgmgr_win = PackageManagerWindow(self)
        self.__pyitool_win = PyinstallerToolWindow(self)
        self.__indexmgr_win = IndexUrlManagerWindow(self)
        self.__pkgdl_win = PackageDownloadWindow(self)
        self.signal_slot_connection()
        self.action_checked(config.app_style)

    def action_checked(self, style: AppStyle):
        if style == AppStyle.Windows:
            self.action_windows.setChecked(True)
            self.action_fusion.setChecked(False)
            self.action_default.setChecked(False)
        elif style == AppStyle.WindowsVista:
            self.action_windows.setChecked(False)
            self.action_fusion.setChecked(False)
            self.action_default.setChecked(True)
        elif style == AppStyle.Fusion:
            self.action_windows.setChecked(False)
            self.action_fusion.setChecked(True)
            self.action_default.setChecked(False)

    def change_appstyle(self, style: AppStyle):
        self.__config.app_style = style
        self.action_checked(style)
        MessageBox("提示", "界面风格设置成功，重启程序生效！").exec_()

    def display(self):
        self.resize(*self.__config.window_size)
        self.showNormal()

    def signal_slot_connection(self):
        self.action_about.triggered.connect(self.__show_about)
        self.pb_pkg_mgr.clicked.connect(self.__pkgmgr_win.display)
        self.pb_pyi_tool.clicked.connect(self.__pyitool_win.display)
        self.pb_index_mgr.clicked.connect(self.__indexmgr_win.display)
        self.pb_pkg_dload.clicked.connect(self.__pkgdl_win.display)
        self.action_fusion.triggered.connect(
            lambda: self.change_appstyle(AppStyle.Fusion)
        )
        self.action_windows.triggered.connect(
            lambda: self.change_appstyle(AppStyle.Windows)
        )
        self.action_default.triggered.connect(
            lambda: self.change_appstyle(AppStyle.WindowsVista)
        )

    def __store_window_size(self):
        if self.isMaximized() or self.isMinimized():
            return
        self.__config.window_size = self.width(), self.height()

    def closeEvent(self, event: QCloseEvent):
        if (
            self.__pkgmgr_win.thread_repo.is_empty()
            and self.__pyitool_win.thread_repo.is_empty()
            and self.__pkgdl_win.thread_repo.is_empty()
        ):
            event.accept()
        else:
            role = MessageBox(
                "警告",
                "有任务正在运行...",
                QMessageBox.Warning,
                (("accept", "强制退出"), ("reject", "取消")),
            ).exec_()
            if role == 0:
                self.__pkgdl_win.thread_repo.kill_all()
                self.__pkgmgr_win.thread_repo.kill_all()
                self.__pyitool_win.thread_repo.kill_all()
                event.accept()
            else:
                event.ignore()
        self.__store_window_size()
        self.__config.save_config()

    @staticmethod
    def __show_about():
        about_path = generate_respath("help", "About.html")
        try:
            with open(about_path, encoding="utf-8") as h:
                info = h.read().replace("0.0.0", VERSION)
                icon = QMessageBox.Information
        except Exception:
            info = f"无法打开<关于>文件：{about_path}，文件已丢失或损坏。"
            icon = QMessageBox.Critical
        MessageBox("关于", info, icon).exec_()


def run_pykit_sysexit_when_close():
    awespykit = QApplication(sys.argv)
    config = MainEntranceConfig()
    awespykit.setWindowIcon(QIcon(":/icon.ico"))
    awespykit.setStyle(AppStyle(config.app_style).name)
    main_entrance = MainEntrance(config)
    main_entrance.display()
    sys.exit(awespykit.exec_())


if __name__ == "__main__":
    run_pykit_sysexit_when_close()



# coding: utf-8

from com import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from ui import *


class GenericOutputWindow(Ui_generic_output, QMainWindow):
    signal_clear_content = pyqtSignal()
    signal_append_line = pyqtSignal(str)

    def __init__(self, parent: QMainWindow, title=None, titlebar=True):
        self.__parent = parent
        super().__init__(parent)
        self.setupUi(self)
        if title is not None:
            self.setWindowTitle(title)
        if not titlebar:
            self.setWindowFlags(Qt.Window | Qt.CustomizeWindowHint)
        self.signal_slot_connection()
        self.__pressed = False
        self.__linkage = Linkage.NoLink
        self.__x = self.x()
        self.__y = self.y()
        self.__w = self.width()
        self.__h = self.height()
        self.installEventFilter(self)
        self.__not_shown_yet = True

    def not_shown_yet(self):
        return self.__not_shown_yet

    @property
    def linkage(self):
        return self.__linkage

    @linkage.setter
    def linkage(self, side: Linkage):
        assert isinstance(side, Linkage)
        self.__linkage = side

    def signal_slot_connection(self):
        self.uiPushButton_close_window.clicked.connect(self.hide)
        self.signal_append_line.connect(self.uiPlainTextEdit_output.appendPlainText)
        self.uiPushButton_clear_content.clicked.connect(
            self.uiPlainTextEdit_output.clear
        )
        self.signal_clear_content.connect(self.uiPlainTextEdit_output.clear)

    def showEvent(self, event: QShowEvent):
        self.__not_shown_yet = False

    def __test_geo(self):
        cur, curf = self.geometry(), self.frameGeometry()
        par, parf = self.__parent.geometry(), self.__parent.frameGeometry()
        if parf.bottom() - 20 <= curf.top() <= parf.bottom() + 20:
            if parf.left() - 20 <= curf.left() <= parf.left() + 20:
                self.__x = parf.left()
                self.__y = parf.bottom() + 1
                self.__w = par.width()
                self.__h = cur.height()
                self.__linkage = Linkage.Top
                return True
        elif parf.top() - 20 < curf.top() < parf.top() + 20:
            if parf.right() - 20 <= curf.left() <= parf.right() + 20:
                self.__x = parf.right() + 1
                self.__y = parf.top()
                self.__w = cur.width()
                self.__h = par.height()
                self.__linkage = Linkage.Left
                return True
            elif parf.left() - 20 <= curf.right() <= parf.left() + 20:
                self.__x = parf.left() - curf.width()
                self.__y = parf.top()
                self.__w = cur.width()
                self.__h = par.height()
                self.__linkage = Linkage.Right
                return True
        self.__linkage = Linkage.NoLink
        return False

    def __make_adsorb(self):
        if not self.__pressed:
            return
        if not self.__test_geo():
            return
        self.move(self.__x, self.__y)
        self.resize(self.__w, self.__h)

    def moveEvent(self, event: QMoveEvent):
        cur_point = event.pos()
        self.__x = cur_point.x()
        self.__y = cur_point.y()

    def resizeEvent(self, event: QResizeEvent):
        cur_size = event.size()
        self.__w = cur_size.width()
        self.__h = cur_size.height()

    def eventFilter(self, sender: QObject, event: QEvent) -> bool:
        if sender == self:
            if event.type() == QEvent.NonClientAreaMouseButtonPress:
                self.__pressed = True
            elif event.type() == QEvent.NonClientAreaMouseButtonRelease:
                self.__pressed = False
            elif event.type() == QEvent.Move or event.type() == QEvent.Resize:
                self.__make_adsorb()
        return super().eventFilter(sender, event)

    def closeEvent(self, event: QCloseEvent):
        self.hide()
        event.ignore()

    def set_geometry(self, point: QPoint, w, h):
        if self.__linkage == Linkage.Left:
            self.__x = point.x() + 1
            self.__y = point.y()
        elif self.__linkage == Linkage.Right:
            self.__x = point.x() - self.frameGeometry().width()
            self.__y = point.y()
        elif self.__linkage == Linkage.Top:
            self.__x = point.x()
            self.__y = point.y() + 1
        else:
            return
        if w is not None:
            self.__w = w
        if h is not None:
            self.__h = h
        self.move(self.__x, self.__y)
        self.resize(self.__w, self.__h)

    def clear_content(self):
        self.signal_clear_content.emit()

    def add_line(self, string):
        self.signal_append_line.emit(string)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'package_manager.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_package_manager(object):
    def setupUi(self, package_manager):
        package_manager.setObjectName("package_manager")
        package_manager.setWindowModality(QtCore.Qt.WindowModal)
        package_manager.resize(900, 620)
        font = QtGui.QFont()
        font.setFamily("Microsoft YaHei UI")
        package_manager.setFont(font)
        self.centralwidget = QtWidgets.QWidget(package_manager)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_6.setContentsMargins(8, 8, 8, 8)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.splitter = QtWidgets.QSplitter(self.centralwidget)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setHandleWidth(6)
        self.splitter.setObjectName("splitter")
        self.layoutWidget = QtWidgets.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSpacing(2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.lb_python_dir = QtWidgets.QLabel(self.layoutWidget)
        self.lb_python_dir.setMinimumSize(QtCore.QSize(0, 25))
        self.lb_python_dir.setObjectName("lb_python_dir")
        self.verticalLayout.addWidget(self.lb_python_dir)
        self.lw_env_list = QtWidgets.QListWidget(self.layoutWidget)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(0, 120, 215))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.HighlightedText, brush)
        brush = QtGui.QBrush(QtGui.QColor(155, 222, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.HighlightedText, brush)
        brush = QtGui.QBrush(QtGui.QColor(190, 190, 190))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.HighlightedText, brush)
        self.lw_env_list.setPalette(palette)
        self.lw_env_list.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.lw_env_list.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.lw_env_list.setObjectName("lw_env_list")
        self.verticalLayout.addWidget(self.lw_env_list)
        self.verticalLayout.setStretch(1, 1)
        self.verticalLayout_3.addLayout(self.verticalLayout)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.btn_autosearch = QtWidgets.QPushButton(self.layoutWidget)
        self.btn_autosearch.setObjectName("btn_autosearch")
        self.horizontalLayout.addWidget(self.btn_autosearch)
        self.btn_addmanully = QtWidgets.QPushButton(self.layoutWidget)
        self.btn_addmanully.setObjectName("btn_addmanully")
        self.horizontalLayout.addWidget(self.btn_addmanully)
        self.btn_delselected = QtWidgets.QPushButton(self.layoutWidget)
        self.btn_delselected.setObjectName("btn_delselected")
        self.horizontalLayout.addWidget(self.btn_delselected)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.verticalLayout_3.setStretch(0, 1)
        self.layoutWidget1 = QtWidgets.QWidget(self.splitter)
        self.layoutWidget1.setObjectName("layoutWidget1")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.layoutWidget1)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setSpacing(2)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.lb_installed_pkgs_info = QtWidgets.QLabel(self.layoutWidget1)
        self.lb_installed_pkgs_info.setObjectName("lb_installed_pkgs_info")
        self.horizontalLayout_4.addWidget(self.lb_installed_pkgs_info)
        spacerItem = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem)
        self.le_search_pkgs_kwd = QtWidgets.QLineEdit(self.layoutWidget1)
        self.le_search_pkgs_kwd.setClearButtonEnabled(True)
        self.le_search_pkgs_kwd.setObjectName("le_search_pkgs_kwd")
        self.horizontalLayout_4.addWidget(self.le_search_pkgs_kwd)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.tw_installed_info = QtWidgets.QTableWidget(self.layoutWidget1)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(0, 120, 215))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.HighlightedText, brush)
        brush = QtGui.QBrush(QtGui.QColor(155, 222, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.HighlightedText, brush)
        brush = QtGui.QBrush(QtGui.QColor(190, 190, 190))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.HighlightedText, brush)
        self.tw_installed_info.setPalette(palette)
        self.tw_installed_info.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tw_installed_info.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tw_installed_info.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.tw_installed_info.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tw_installed_info.setCornerButtonEnabled(False)
        self.tw_installed_info.setColumnCount(4)
        self.tw_installed_info.setObjectName("tw_installed_info")
        self.tw_installed_info.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tw_installed_info.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tw_installed_info.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tw_installed_info.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tw_installed_info.setHorizontalHeaderItem(3, item)
        self.tw_installed_info.horizontalHeader().setHighlightSections(False)
        self.tw_installed_info.verticalHeader().setHighlightSections(False)
        self.verticalLayout_2.addWidget(self.tw_installed_info)
        self.verticalLayout_2.setStretch(1, 1)
        self.verticalLayout_4.addLayout(self.verticalLayout_2)
        self.glo_table_btns = QtWidgets.QGridLayout()
        self.glo_table_btns.setObjectName("glo_table_btns")
        self.btn_upgrade_package = QtWidgets.QPushButton(self.layoutWidget1)
        self.btn_upgrade_package.setObjectName("btn_upgrade_package")
        self.glo_table_btns.addWidget(self.btn_upgrade_package, 1, 2, 1, 1)
        self.btn_check_for_updates = QtWidgets.QPushButton(self.layoutWidget1)
        self.btn_check_for_updates.setObjectName("btn_check_for_updates")
        self.glo_table_btns.addWidget(self.btn_check_for_updates, 0, 3, 1, 1)
        self.btn_install_package = QtWidgets.QPushButton(self.layoutWidget1)
        self.btn_install_package.setObjectName("btn_install_package")
        self.glo_table_btns.addWidget(self.btn_install_package, 1, 0, 1, 1)
        self.lb_num_selected_items = QtWidgets.QLabel(self.layoutWidget1)
        font = QtGui.QFont()
        font.setFamily("Microsoft YaHei UI")
        font.setPointSize(8)
        self.lb_num_selected_items.setFont(font)
        self.lb_num_selected_items.setObjectName("lb_num_selected_items")
        self.glo_table_btns.addWidget(self.lb_num_selected_items, 0, 0, 1, 1)
        self.btn_uninstall_package = QtWidgets.QPushButton(self.layoutWidget1)
        self.btn_uninstall_package.setObjectName("btn_uninstall_package")
        self.glo_table_btns.addWidget(self.btn_uninstall_package, 1, 1, 1, 1)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_2 = QtWidgets.QLabel(self.layoutWidget1)
        self.label_2.setText("")
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_2.addWidget(self.label_2)
        self.cb_check_uncheck_all = QtWidgets.QCheckBox(self.layoutWidget1)
        self.cb_check_uncheck_all.setObjectName("cb_check_uncheck_all")
        self.horizontalLayout_2.addWidget(self.cb_check_uncheck_all)
        self.glo_table_btns.addLayout(self.horizontalLayout_2, 0, 1, 1, 1)
        self.uiPushButton_show_output = QtWidgets.QPushButton(self.layoutWidget1)
        self.uiPushButton_show_output.setObjectName("uiPushButton_show_output")
        self.glo_table_btns.addWidget(self.uiPushButton_show_output, 0, 2, 1, 1)
        self.btn_upgrade_all = QtWidgets.QPushButton(self.layoutWidget1)
        self.btn_upgrade_all.setObjectName("btn_upgrade_all")
        self.glo_table_btns.addWidget(self.btn_upgrade_all, 1, 3, 1, 1)
        self.verticalLayout_4.addLayout(self.glo_table_btns)
        self.verticalLayout_4.setStretch(0, 1)
        self.verticalLayout_5.addWidget(self.splitter)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setContentsMargins(-1, -1, 4, -1)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.lb_loading_gif = QtWidgets.QLabel(self.centralwidget)
        font = QtGui.QFont()
        font.setFamily("Microsoft YaHei UI")
        font.setPointSize(8)
        self.lb_loading_gif.setFont(font)
        self.lb_loading_gif.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lb_loading_gif.setObjectName("lb_loading_gif")
        self.horizontalLayout_3.addWidget(self.lb_loading_gif)
        self.lb_loading_tip = QtWidgets.QLabel(self.centralwidget)
        font = QtGui.QFont()
        font.setFamily("Microsoft YaHei UI")
        font.setPointSize(8)
        self.lb_loading_tip.setFont(font)
        self.lb_loading_tip.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.lb_loading_tip.setObjectName("lb_loading_tip")
        self.horizontalLayout_3.addWidget(self.lb_loading_tip)
        self.horizontalLayout_3.setStretch(0, 1)
        self.verticalLayout_5.addLayout(self.horizontalLayout_3)
        self.verticalLayout_5.setStretch(0, 1)
        self.verticalLayout_6.addLayout(self.verticalLayout_5)
        package_manager.setCentralWidget(self.centralwidget)

        self.retranslateUi(package_manager)
        QtCore.QMetaObject.connectSlotsByName(package_manager)

    def retranslateUi(self, package_manager):
        _translate = QtCore.QCoreApplication.translate
        package_manager.setWindowTitle(_translate("package_manager", "包管理器"))
        self.lb_python_dir.setToolTip(_translate("package_manager", "显示当前搜索到的/用户添加的 Python 目录路径。"))
        self.lb_python_dir.setText(_translate("package_manager", "Python 环境："))
        self.lw_env_list.setToolTip(_translate("package_manager", "使用鼠标左键点击加载选中的 Python 环境已安装的包信息列表，使用鼠标右键点击或使用上下方向键选择则不加载。"))
        self.btn_autosearch.setToolTip(_translate("package_manager", "自动搜索常用安装位置中的 Python 目录并将路径添加到 Python 环境列表保存。"))
        self.btn_autosearch.setText(_translate("package_manager", "自动搜索"))
        self.btn_addmanully.setToolTip(_translate("package_manager", "手动将 Python 目录路径添加到本 Python 环境列表以便下次使用。"))
        self.btn_addmanully.setText(_translate("package_manager", "手动添加"))
        self.btn_delselected.setToolTip(_translate("package_manager", "将 Python 环境列表中的选中项移除(不会删除本机 Python 目录)。"))
        self.btn_delselected.setText(_translate("package_manager", "移除选中项"))
        self.lb_installed_pkgs_info.setToolTip(_translate("package_manager", "选中的 Python 环境中已安装的包名、当前版本、最新版本、安装状态信息。"))
        self.lb_installed_pkgs_info.setText(_translate("package_manager", "已安装的包信息："))
        self.le_search_pkgs_kwd.setPlaceholderText(_translate("package_manager", "搜索"))
        item = self.tw_installed_info.horizontalHeaderItem(0)
        item.setText(_translate("package_manager", "名称"))
        item = self.tw_installed_info.horizontalHeaderItem(1)
        item.setText(_translate("package_manager", "当前版本"))
        item = self.tw_installed_info.horizontalHeaderItem(2)
        item.setText(_translate("package_manager", "最新版本"))
        item = self.tw_installed_info.horizontalHeaderItem(3)
        item.setText(_translate("package_manager", "状态"))
        self.btn_upgrade_package.setToolTip(_translate("package_manager", "升级表格中被选中的包。\n"
"注意，被选中的包如果没有新版本则不升级，支持多选。"))
        self.btn_upgrade_package.setText(_translate("package_manager", "升级"))
        self.btn_check_for_updates.setToolTip(_translate("package_manager", "检查选中的 Python 环境中的所有模块的最新版本，有新版本则在\"最新版本\"列中显示版本号。"))
        self.btn_check_for_updates.setText(_translate("package_manager", "检查更新"))
        self.btn_install_package.setToolTip(_translate("package_manager", "将输入的包名安装到选中的 Python 环境中。\n"
"多个包名请用空格隔开，支持输入与 pip 命令 install 选项相同的参数。"))
        self.btn_install_package.setText(_translate("package_manager", "安装"))
        self.btn_uninstall_package.setToolTip(_translate("package_manager", "将表格中选中的包卸载(仅从选中的 Python 环境中卸载)。\n"
"注意，卸载时表格中的条目支持多选。"))
        self.btn_uninstall_package.setText(_translate("package_manager", "卸载"))
        self.cb_check_uncheck_all.setText(_translate("package_manager", "全选"))
        self.uiPushButton_show_output.setText(_translate("package_manager", "输出窗口"))
        self.btn_upgrade_all.setToolTip(_translate("package_manager", "升级表格中列出的所有显示有新版本的包。\n"
"使用此功能前请先点击\"检查更新\"按钮以检查是否有新版本，再点击此按钮升级。"))
        self.btn_upgrade_all.setText(_translate("package_manager", "升级全部"))



# coding: utf-8

__doc__ = """包含一些继承自默认Qt控件的自定义行为控件。"""

import os
from typing import List

from com.enums import Accept
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *


class LineEdit(QLineEdit):
    def __init__(self, accept=None, ext_filter=None):
        super().__init__()
        self.setContextMenuPolicy(Qt.NoContextMenu)
        if accept is None:
            self.__accept = Accept.Dir
        else:
            self.__accept = accept
        if ext_filter is None:
            self.__filter = set()
        else:
            self.__filter = ext_filter
            assert isinstance(ext_filter, set)
        self.__drag_temp = ""

    @property
    def local_path(self):
        return self.text().strip()

    def dragEnterEvent(self, event: QDragEnterEvent):
        if event.mimeData().hasUrls():
            if self.__accept == Accept.Dir:
                event.accept()
            elif self.__accept == Accept.File:
                self.__drag_temp = os.path.realpath(
                    event.mimeData().urls()[0].toLocalFile()
                )
                if (
                    not self.__filter
                    or os.path.splitext(self.__drag_temp)[1] in self.__filter
                ):
                    event.accept()
                else:
                    event.ignore()
            else:
                event.ignore()
        else:
            event.ignore()

    def dropEvent(self, event: QDropEvent):
        if not self.__drag_temp:
            self.__drag_temp = os.path.realpath(
                event.mimeData().urls()[0].toLocalFile()
            )
        if self.__accept == Accept.File and os.path.isfile(self.__drag_temp):
            self.setText(self.__drag_temp)
        elif self.__accept == Accept.Dir and os.path.isdir(self.__drag_temp):
            self.setText(self.__drag_temp)


class TextEdit(QTextEdit):
    def __init__(self, accept=None, ext_filter=None):
        super().__init__()
        self.setLineWrapMode(QTextEdit.NoWrap)
        self.setContextMenuPolicy(Qt.NoContextMenu)
        if accept is None:
            self.__accept = Accept.File
        else:
            self.__accept = accept
        if ext_filter is None:
            self.__filter = set()
        else:
            self.__filter = ext_filter
            assert isinstance(ext_filter, set)
        self.__drag_temp = list()

    @property
    def local_paths(self):
        file_dir_paths = self.toPlainText().split("\n")
        if self.__accept == Accept.Dir:
            return [p for p in file_dir_paths if os.path.isdir(p)]
        if self.__accept == Accept.File:
            return [p for p in file_dir_paths if os.path.isfile(p)]
        return list()

    def __stash_from_urls(self, urls: List[QUrl]):
        self.__drag_temp.clear()
        for file_or_dir in (path.toLocalFile() for path in urls):
            file_or_dir = os.path.realpath(file_or_dir)
            if os.path.isfile(file_or_dir):
                self.__drag_temp.append(file_or_dir)
                continue
            self.__drag_temp.append(file_or_dir)
            for root, _, files in os.walk(file_or_dir):
                self.__drag_temp.extend(
                    os.path.join(root, filename) for filename in files
                )

    def dragEnterEvent(self, event: QDragEnterEvent):
        self.__drag_temp.clear()
        if event.mimeData().hasUrls():
            if self.__accept == Accept.File:
                self.__stash_from_urls(event.mimeData().urls())
                if not self.__filter or set(
                    os.path.splitext(fp)[1]
                    for fp in self.__drag_temp
                    if os.path.isfile(fp)
                ).issubset(self.__filter):
                    event.accept()
                else:
                    event.ignore()
            elif self.__accept == Accept.Dir:
                event.accept()
            else:
                event.ignore()
            if not self.toPlainText().endswith("\n"):
                self.append("")
        else:
            event.ignore()

    def dropEvent(self, event: QDropEvent):
        cur_text = self.toPlainText()
        super().dropEvent(event)
        if not self.__drag_temp:
            self.__stash_from_urls(event.mimeData().urls())
        if self.__accept == Accept.File:
            self.setText(
                cur_text + "\n".join(p for p in self.__drag_temp if os.path.isfile(p))
            )
        elif self.__accept == Accept.Dir:
            self.setText(
                cur_text + "\n".join(p for p in self.__drag_temp if os.path.isdir(p))
            )
        else:
            self.setText("")
        self.verticalScrollBar().setValue(self.verticalScrollBar().maximumHeight())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'generic_output.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_generic_output(object):
    def setupUi(self, generic_output):
        generic_output.setObjectName("generic_output")
        generic_output.resize(350, 500)
        font = QtGui.QFont()
        font.setFamily("Microsoft YaHei UI")
        generic_output.setFont(font)
        self.centralwidget = QtWidgets.QWidget(generic_output)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setContentsMargins(4, 4, 4, 4)
        self.verticalLayout.setSpacing(2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiPlainTextEdit_output = QtWidgets.QPlainTextEdit(self.centralwidget)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(10)
        self.uiPlainTextEdit_output.setFont(font)
        self.uiPlainTextEdit_output.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.uiPlainTextEdit_output.setReadOnly(True)
        self.uiPlainTextEdit_output.setPlainText("")
        self.uiPlainTextEdit_output.setObjectName("uiPlainTextEdit_output")
        self.verticalLayout.addWidget(self.uiPlainTextEdit_output)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.uiPushButton_close_window = QtWidgets.QPushButton(self.centralwidget)
        self.uiPushButton_close_window.setObjectName("uiPushButton_close_window")
        self.horizontalLayout.addWidget(self.uiPushButton_close_window)
        self.uiPushButton_clear_content = QtWidgets.QPushButton(self.centralwidget)
        self.uiPushButton_clear_content.setObjectName("uiPushButton_clear_content")
        self.horizontalLayout.addWidget(self.uiPushButton_clear_content)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.verticalLayout.setStretch(0, 1)
        generic_output.setCentralWidget(self.centralwidget)
        self.action123 = QtWidgets.QAction(generic_output)
        font = QtGui.QFont()
        font.setFamily("Microsoft YaHei UI")
        self.action123.setFont(font)
        self.action123.setObjectName("action123")
        self.uiAction_bg_white = QtWidgets.QAction(generic_output)
        self.uiAction_bg_white.setObjectName("uiAction_bg_white")
        self.uiAction_bg_black = QtWidgets.QAction(generic_output)
        self.uiAction_bg_black.setObjectName("uiAction_bg_black")

        self.retranslateUi(generic_output)
        QtCore.QMetaObject.connectSlotsByName(generic_output)

    def retranslateUi(self, generic_output):
        _translate = QtCore.QCoreApplication.translate
        generic_output.setWindowTitle(_translate("generic_output", "控制台输出内容"))
        self.uiPushButton_close_window.setText(_translate("generic_output", "关闭窗口"))
        self.uiPushButton_clear_content.setText(_translate("generic_output", "清空内容"))
        self.action123.setText(_translate("generic_output", "字体颜色"))
        self.uiAction_bg_white.setText(_translate("generic_output", "白色"))
        self.uiAction_bg_black.setText(_translate("generic_output", "黑色"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'package_download.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_package_download(object):
    def setupUi(self, package_download):
        package_download.setObjectName("package_download")
        package_download.setWindowModality(QtCore.Qt.WindowModal)
        package_download.resize(620, 658)
        font = QtGui.QFont()
        font.setFamily("Microsoft YaHei UI")
        package_download.setFont(font)
        package_download.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.centralwidget = QtWidgets.QWidget(package_download)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_14 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_14.setContentsMargins(8, 8, 8, 8)
        self.verticalLayout_14.setObjectName("verticalLayout_14")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout()
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout()
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout()
        self.verticalLayout_8.setSpacing(2)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setObjectName("label")
        self.verticalLayout_8.addWidget(self.label)
        self.pte_package_names = QtWidgets.QPlainTextEdit(self.centralwidget)
        self.pte_package_names.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.pte_package_names.setObjectName("pte_package_names")
        self.verticalLayout_8.addWidget(self.pte_package_names)
        self.verticalLayout_10.addLayout(self.verticalLayout_8)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.pb_clear_package_names = QtWidgets.QPushButton(self.centralwidget)
        self.pb_clear_package_names.setObjectName("pb_clear_package_names")
        self.horizontalLayout_3.addWidget(self.pb_clear_package_names)
        self.pb_load_from_text = QtWidgets.QPushButton(self.centralwidget)
        self.pb_load_from_text.setObjectName("pb_load_from_text")
        self.horizontalLayout_3.addWidget(self.pb_load_from_text)
        self.pb_save_as_text = QtWidgets.QPushButton(self.centralwidget)
        self.pb_save_as_text.setObjectName("pb_save_as_text")
        self.horizontalLayout_3.addWidget(self.pb_save_as_text)
        self.horizontalLayout_3.setStretch(1, 1)
        self.horizontalLayout_3.setStretch(2, 1)
        self.verticalLayout_10.addLayout(self.horizontalLayout_3)
        self.verticalLayout_11.addLayout(self.verticalLayout_10)
        self.line_7 = QtWidgets.QFrame(self.centralwidget)
        self.line_7.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_7.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_7.setObjectName("line_7")
        self.verticalLayout_11.addWidget(self.line_7)
        self.verticalLayout_9 = QtWidgets.QVBoxLayout()
        self.verticalLayout_9.setSpacing(2)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.label_8 = QtWidgets.QLabel(self.centralwidget)
        self.label_8.setObjectName("label_8")
        self.verticalLayout_9.addWidget(self.label_8)
        self.cmb_derived_from = QtWidgets.QComboBox(self.centralwidget)
        self.cmb_derived_from.setObjectName("cmb_derived_from")
        self.verticalLayout_9.addWidget(self.cmb_derived_from)
        self.verticalLayout_11.addLayout(self.verticalLayout_9)
        self.line_8 = QtWidgets.QFrame(self.centralwidget)
        self.line_8.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_8.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_8.setObjectName("line_8")
        self.verticalLayout_11.addWidget(self.line_8)
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(8)
        self.label_2.setFont(font)
        self.label_2.setWordWrap(True)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_11.addWidget(self.label_2)
        self.horizontalLayout_6.addLayout(self.verticalLayout_11)
        self.verticalLayout_13 = QtWidgets.QVBoxLayout()
        self.verticalLayout_13.setObjectName("verticalLayout_13")
        self.groupBox = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_12 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout()
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.cb_download_deps = QtWidgets.QCheckBox(self.groupBox)
        self.cb_download_deps.setObjectName("cb_download_deps")
        self.verticalLayout_7.addWidget(self.cb_download_deps)
        self.groupBox_2 = QtWidgets.QGroupBox(self.groupBox)
        self.groupBox_2.setTitle("")
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.rb_unlimited = QtWidgets.QRadioButton(self.groupBox_2)
        self.rb_unlimited.setObjectName("rb_unlimited")
        self.verticalLayout_5.addWidget(self.rb_unlimited)
        self.rb_no_binary = QtWidgets.QRadioButton(self.groupBox_2)
        self.rb_no_binary.setObjectName("rb_no_binary")
        self.verticalLayout_5.addWidget(self.rb_no_binary)
        self.rb_only_binary = QtWidgets.QRadioButton(self.groupBox_2)
        self.rb_only_binary.setObjectName("rb_only_binary")
        self.verticalLayout_5.addWidget(self.rb_only_binary)
        self.rb_prefer_binary = QtWidgets.QRadioButton(self.groupBox_2)
        self.rb_prefer_binary.setObjectName("rb_prefer_binary")
        self.verticalLayout_5.addWidget(self.rb_prefer_binary)
        self.verticalLayout_6.addLayout(self.verticalLayout_5)
        self.verticalLayout_7.addWidget(self.groupBox_2)
        self.cb_include_pre = QtWidgets.QCheckBox(self.groupBox)
        self.cb_include_pre.setObjectName("cb_include_pre")
        self.verticalLayout_7.addWidget(self.cb_include_pre)
        self.cb_ignore_requires_python = QtWidgets.QCheckBox(self.groupBox)
        self.cb_ignore_requires_python.setObjectName("cb_ignore_requires_python")
        self.verticalLayout_7.addWidget(self.cb_ignore_requires_python)
        self.line = QtWidgets.QFrame(self.groupBox)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_7.addWidget(self.line)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setSpacing(2)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_3 = QtWidgets.QLabel(self.groupBox)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_5.addWidget(self.label_3)
        self.pb_save_to = QtWidgets.QPushButton(self.groupBox)
        self.pb_save_to.setObjectName("pb_save_to")
        self.horizontalLayout_5.addWidget(self.pb_save_to)
        self.horizontalLayout_5.setStretch(0, 9)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.le_save_to = QtWidgets.QLineEdit(self.groupBox)
        self.le_save_to.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.le_save_to.setObjectName("le_save_to")
        self.verticalLayout_2.addWidget(self.le_save_to)
        self.verticalLayout_7.addLayout(self.verticalLayout_2)
        self.line_2 = QtWidgets.QFrame(self.groupBox)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_7.addWidget(self.line_2)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSpacing(2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_4 = QtWidgets.QLabel(self.groupBox)
        self.label_4.setObjectName("label_4")
        self.verticalLayout.addWidget(self.label_4)
        self.le_platform = QtWidgets.QLineEdit(self.groupBox)
        self.le_platform.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.le_platform.setObjectName("le_platform")
        self.verticalLayout.addWidget(self.le_platform)
        self.verticalLayout_7.addLayout(self.verticalLayout)
        self.line_3 = QtWidgets.QFrame(self.groupBox)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.verticalLayout_7.addWidget(self.line_3)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_5 = QtWidgets.QLabel(self.groupBox)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout.addWidget(self.label_5)
        self.le_python_version = QtWidgets.QLineEdit(self.groupBox)
        self.le_python_version.setMinimumSize(QtCore.QSize(180, 0))
        self.le_python_version.setMaximumSize(QtCore.QSize(180, 16777215))
        self.le_python_version.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.le_python_version.setObjectName("le_python_version")
        self.horizontalLayout.addWidget(self.le_python_version)
        self.horizontalLayout.setStretch(1, 9)
        self.verticalLayout_7.addLayout(self.horizontalLayout)
        self.line_4 = QtWidgets.QFrame(self.groupBox)
        self.line_4.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_4.setObjectName("line_4")
        self.verticalLayout_7.addWidget(self.line_4)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_6 = QtWidgets.QLabel(self.groupBox)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_2.addWidget(self.label_6)
        self.cmb_implementation = QtWidgets.QComboBox(self.groupBox)
        self.cmb_implementation.setMinimumSize(QtCore.QSize(180, 0))
        self.cmb_implementation.setMaximumSize(QtCore.QSize(180, 16777215))
        self.cmb_implementation.setObjectName("cmb_implementation")
        self.cmb_implementation.addItem("")
        self.cmb_implementation.setItemText(0, "")
        self.cmb_implementation.addItem("")
        self.cmb_implementation.addItem("")
        self.cmb_implementation.addItem("")
        self.cmb_implementation.addItem("")
        self.cmb_implementation.addItem("")
        self.horizontalLayout_2.addWidget(self.cmb_implementation)
        self.horizontalLayout_2.setStretch(1, 9)
        self.verticalLayout_7.addLayout(self.horizontalLayout_2)
        self.line_5 = QtWidgets.QFrame(self.groupBox)
        self.line_5.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_5.setObjectName("line_5")
        self.verticalLayout_7.addWidget(self.line_5)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setSpacing(2)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label_7 = QtWidgets.QLabel(self.groupBox)
        self.label_7.setObjectName("label_7")
        self.verticalLayout_3.addWidget(self.label_7)
        self.le_abis = QtWidgets.QLineEdit(self.groupBox)
        self.le_abis.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.le_abis.setObjectName("le_abis")
        self.verticalLayout_3.addWidget(self.le_abis)
        self.verticalLayout_7.addLayout(self.verticalLayout_3)
        self.line_6 = QtWidgets.QFrame(self.groupBox)
        self.line_6.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_6.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_6.setObjectName("line_6")
        self.verticalLayout_7.addWidget(self.line_6)
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setSpacing(2)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.cb_use_index_url = QtWidgets.QCheckBox(self.groupBox)
        self.cb_use_index_url.setObjectName("cb_use_index_url")
        self.verticalLayout_4.addWidget(self.cb_use_index_url)
        self.le_index_url = QtWidgets.QLineEdit(self.groupBox)
        self.le_index_url.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.le_index_url.setObjectName("le_index_url")
        self.verticalLayout_4.addWidget(self.le_index_url)
        self.verticalLayout_7.addLayout(self.verticalLayout_4)
        self.verticalLayout_12.addLayout(self.verticalLayout_7)
        self.verticalLayout_13.addWidget(self.groupBox)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_13.addItem(spacerItem)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.pb_show_dl_list = QtWidgets.QPushButton(self.centralwidget)
        self.pb_show_dl_list.setMinimumSize(QtCore.QSize(0, 50))
        font = QtGui.QFont()
        font.setFamily("Microsoft YaHei UI")
        font.setPointSize(10)
        self.pb_show_dl_list.setFont(font)
        self.pb_show_dl_list.setObjectName("pb_show_dl_list")
        self.horizontalLayout_4.addWidget(self.pb_show_dl_list)
        self.pb_start_download = QtWidgets.QPushButton(self.centralwidget)
        self.pb_start_download.setMinimumSize(QtCore.QSize(0, 50))
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.ButtonText, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.ButtonText, brush)
        brush = QtGui.QBrush(QtGui.QColor(120, 120, 120))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.ButtonText, brush)
        self.pb_start_download.setPalette(palette)
        font = QtGui.QFont()
        font.setFamily("Microsoft YaHei UI")
        font.setPointSize(12)
        self.pb_start_download.setFont(font)
        self.pb_start_download.setObjectName("pb_start_download")
        self.horizontalLayout_4.addWidget(self.pb_start_download)
        self.horizontalLayout_4.setStretch(1, 1)
        self.verticalLayout_13.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_6.addLayout(self.verticalLayout_13)
        self.verticalLayout_14.addLayout(self.horizontalLayout_6)
        package_download.setCentralWidget(self.centralwidget)

        self.retranslateUi(package_download)
        QtCore.QMetaObject.connectSlotsByName(package_download)

    def retranslateUi(self, package_download):
        _translate = QtCore.QCoreApplication.translate
        package_download.setWindowTitle(_translate("package_download", "模块安装包下载"))
        self.label.setToolTip(_translate("package_download", "需要下载的模块名称，每行一个名称。"))
        self.label.setText(_translate("package_download", "名称："))
        self.pte_package_names.setToolTip(_translate("package_download", "需要下载的模块名称，每行一个名称。"))
        self.pb_clear_package_names.setToolTip(_translate("package_download", "清空名称编辑区文字。"))
        self.pb_clear_package_names.setText(_translate("package_download", "清空"))
        self.pb_load_from_text.setToolTip(_translate("package_download", "从文本文件加载名称到名称编辑区。"))
        self.pb_load_from_text.setText(_translate("package_download", "从文件加载名称"))
        self.pb_save_as_text.setToolTip(_translate("package_download", "将名称编辑区的文字保存到文本文件。"))
        self.pb_save_as_text.setText(_translate("package_download", "名称保存到文件"))
        self.label_8.setToolTip(_translate("package_download", "下载安装包时需调用的 Python 环境。\n"
"右侧留空的下载条件，下载时会使用从此环境派生的条件。"))
        self.label_8.setText(_translate("package_download", "下载条件默认派生自："))
        self.cmb_derived_from.setToolTip(_translate("package_download", "下载安装包时需调用的 Python 环境。\n"
"右侧留空的下载条件，下载时会使用从此环境派生的条件。"))
        self.label_2.setText(_translate("package_download", "名称后支持跟随以下符号限定要下载的版本：\n"
"\"==\"、\">=\"、\"<=\"、\">\"、\"<\"、\",\"\n"
"每行一个名称，名称和限定符中不允许出现空格。\n"
"例如：fastpip>=0.6.2,<0.10.0\n"
"如果限制条件留空，则下载兼容所选环境的安装包。"))
        self.groupBox.setTitle(_translate("package_download", "下载条件"))
        self.cb_download_deps.setToolTip(_translate("package_download", "对于名称编辑区中的每一个需要下载的模块，是否同时下载模块的依赖库。"))
        self.cb_download_deps.setText(_translate("package_download", "下载需要下载的包的依赖"))
        self.rb_unlimited.setToolTip(_translate("package_download", "不限制下载二进制包或者源代码包。"))
        self.rb_unlimited.setText(_translate("package_download", "不限制下载的包类型"))
        self.rb_no_binary.setToolTip(_translate("package_download", "仅下载包的源代码安装包。"))
        self.rb_no_binary.setText(_translate("package_download", "仅选择源代码包"))
        self.rb_only_binary.setToolTip(_translate("package_download", "仅下载包的二进制安装包。"))
        self.rb_only_binary.setText(_translate("package_download", "仅选择二进制包"))
        self.rb_prefer_binary.setToolTip(_translate("package_download", "对于既有二进制安装包又有源代码安装包的模块，\n"
"如果较新版本没有发布二进制安装包，则宁愿下载较旧版本的二进制安装包。"))
        self.rb_prefer_binary.setText(_translate("package_download", "宁选择旧二进制包而非新源代码包"))
        self.cb_include_pre.setToolTip(_translate("package_download", "如果模块的最新版本是预发行版或者是开发版，\n"
"也下载这些版本，否则只下载模块的最新稳定版本。"))
        self.cb_include_pre.setText(_translate("package_download", "包括预发行版和开发版"))
        self.cb_ignore_requires_python.setToolTip(_translate("package_download", "对于那些对 Python 版本有限制要求的模块，是否忽略其限制要求。"))
        self.cb_ignore_requires_python.setText(_translate("package_download", "忽略包的 Python 版本限制"))
        self.label_3.setToolTip(_translate("package_download", "下载的模块安装包的保存路径。"))
        self.label_3.setText(_translate("package_download", "下载至："))
        self.pb_save_to.setToolTip(_translate("package_download", "选择下载的模块安装包的保存路径。"))
        self.pb_save_to.setText(_translate("package_download", "选择目录"))
        self.le_save_to.setToolTip(_translate("package_download", "下载的模块安装包的保存路径。"))
        self.label_4.setToolTip(_translate("package_download", "是否只下载兼容此处列出的平台的模块安装包。"))
        self.label_4.setText(_translate("package_download", "兼容平台(空格分隔)："))
        self.le_platform.setToolTip(_translate("package_download", "是否只下载兼容此处列出的平台的模块安装包。"))
        self.label_5.setToolTip(_translate("package_download", "是否只下载兼容此处列出的 Python 版本的模块安装包。"))
        self.label_5.setText(_translate("package_download", "兼容 Python 版本："))
        self.le_python_version.setToolTip(_translate("package_download", "是否只下载兼容此处列出的 Python 版本的模块安装包。"))
        self.label_6.setToolTip(_translate("package_download", "是否只下载兼容此处列出的 Python 实现的模块安装包。"))
        self.label_6.setText(_translate("package_download", "兼容解释器实现："))
        self.cmb_implementation.setToolTip(_translate("package_download", "是否只下载兼容此处列出的 Python 实现的模块安装包。"))
        self.cmb_implementation.setItemText(1, _translate("package_download", "无特定实现"))
        self.cmb_implementation.setItemText(2, _translate("package_download", "CPython"))
        self.cmb_implementation.setItemText(3, _translate("package_download", "Jython"))
        self.cmb_implementation.setItemText(4, _translate("package_download", "PyPy"))
        self.cmb_implementation.setItemText(5, _translate("package_download", "IronPython"))
        self.label_7.setToolTip(_translate("package_download", "是否只下载兼容此处列出的 Python ABI的模块安装包。"))
        self.label_7.setText(_translate("package_download", "兼容ABI(空格分隔)："))
        self.le_abis.setToolTip(_translate("package_download", "是否只下载兼容此处列出的 Python ABI的模块安装包。"))
        self.cb_use_index_url.setToolTip(_translate("package_download", "是否从临时镜像源下载安装包。\n"
"如果不勾选此选项，则默认从系统已设置的镜像源地址下载。"))
        self.cb_use_index_url.setText(_translate("package_download", "使用临时镜像源："))
        self.le_index_url.setToolTip(_translate("package_download", "临时镜像源地址。"))
        self.pb_show_dl_list.setText(_translate("package_download", "下载列表"))
        self.pb_start_download.setText(_translate("package_download", "开始下载"))



# coding: utf-8

import json
import os
import os.path as op
import sys
from pathlib import Path

_runs_in_bundle_mode = getattr(sys, "frozen", False)
if _runs_in_bundle_mode:
    _res_root = sys._MEIPASS
else:
    _res_root = op.dirname(op.dirname(op.abspath(__file__)))
_appdata_local_folder = os.getenv("LOCALAPPDATA")
if not _appdata_local_folder:
    if _runs_in_bundle_mode:
        config_root = op.dirname(sys.executable)
    else:
        config_root = _res_root
else:
    config_root = op.join(_appdata_local_folder, "Awespykit")
config_root = op.join(config_root, "config")


def generate_respath(*p):
    """
    用于在不同运行环境之下获取正确的资源文件路径

    不同的运行环境：Python 源代码运行、Pyinstaller 打包为单文件运行/打包为单目录运行
    """
    return op.join(_res_root, *p)


class AbstractConfig(dict):
    root = Path(config_root)

    def __init__(self, fname):
        super().__init__()
        self.__cfg = self.root.joinpath(fname)
        self.__load_json()

    def save_config(self):
        try:
            with open(self.__cfg, "wt", encoding="utf-8") as f:
                json.dump(self, f, ensure_ascii=False)
        except Exception:
            pass

    def __load_json(self):
        """如果 __cfg 文件无法读取则返回空字典"""
        try:
            if not self.root.exists():
                self.root.mkdir(parents=True)
        except Exception:
            return
        if not self.__cfg.exists():
            try:
                with open(self.__cfg, "wt", encoding="utf-8") as f:
                    json.dump(dict(), f)
            except Exception:
                return
        else:
            try:
                with open(self.__cfg, "rt", encoding="utf-8") as f:
                    self.update(json.load(f))
            except Exception:
                return



# coding: utf-8

from .main import (
    QThreadModel,
    ThreadRepo,
    check_index_url,
    check_py_path,
    clean_index_urls,
    clean_py_paths,
    get_cmd_out,
    loop_install,
    loop_uninstall,
    open_explorer,
)

__all__ = [
    "QThreadModel",
    "ThreadRepo",
    "check_index_url",
    "check_py_path",
    "clean_index_urls",
    "clean_py_paths",
    "get_cmd_out",
    "loop_install",
    "loop_uninstall",
    "open_explorer",
]



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'query_panel.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_query_panel(object):
    def setupUi(self, query_panel):
        query_panel.setObjectName("query_panel")
        query_panel.setWindowModality(QtCore.Qt.WindowModal)
        query_panel.resize(330, 330)
        font = QtGui.QFont()
        font.setFamily("Microsoft YaHei UI")
        query_panel.setFont(font)
        self.centralwidget = QtWidgets.QWidget(query_panel)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_3.setContentsMargins(8, 8, 8, 8)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.uiLineEdit_input_name = QtWidgets.QLineEdit(self.centralwidget)
        self.uiLineEdit_input_name.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.uiLineEdit_input_name.setObjectName("uiLineEdit_input_name")
        self.horizontalLayout.addWidget(self.uiLineEdit_input_name)
        self.uiPushButton_query = QtWidgets.QPushButton(self.centralwidget)
        self.uiPushButton_query.setObjectName("uiPushButton_query")
        self.horizontalLayout.addWidget(self.uiPushButton_query)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.uiRadioButton_pkg2import = QtWidgets.QRadioButton(self.centralwidget)
        self.uiRadioButton_pkg2import.setChecked(True)
        self.uiRadioButton_pkg2import.setObjectName("uiRadioButton_pkg2import")
        self.horizontalLayout_2.addWidget(self.uiRadioButton_pkg2import)
        self.uiRadioButton_import2pkg = QtWidgets.QRadioButton(self.centralwidget)
        self.uiRadioButton_import2pkg.setObjectName("uiRadioButton_import2pkg")
        self.horizontalLayout_2.addWidget(self.uiRadioButton_import2pkg)
        self.uiCheckBox_case_sensitive = QtWidgets.QCheckBox(self.centralwidget)
        self.uiCheckBox_case_sensitive.setObjectName("uiCheckBox_case_sensitive")
        self.horizontalLayout_2.addWidget(self.uiCheckBox_case_sensitive)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.line = QtWidgets.QFrame(self.centralwidget)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_3.addWidget(self.line)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setSpacing(2)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setObjectName("label")
        self.verticalLayout_2.addWidget(self.label)
        self.uiLabel_query_environ = QtWidgets.QLabel(self.centralwidget)
        self.uiLabel_query_environ.setFrameShape(QtWidgets.QFrame.Box)
        self.uiLabel_query_environ.setObjectName("uiLabel_query_environ")
        self.verticalLayout_2.addWidget(self.uiLabel_query_environ)
        self.verticalLayout_3.addLayout(self.verticalLayout_2)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSpacing(2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiLabel_query_result = QtWidgets.QLabel(self.centralwidget)
        self.uiLabel_query_result.setObjectName("uiLabel_query_result")
        self.verticalLayout.addWidget(self.uiLabel_query_result)
        self.uiPlainTextEdit_query_result = QtWidgets.QPlainTextEdit(self.centralwidget)
        self.uiPlainTextEdit_query_result.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.uiPlainTextEdit_query_result.setObjectName("uiPlainTextEdit_query_result")
        self.verticalLayout.addWidget(self.uiPlainTextEdit_query_result)
        self.verticalLayout_3.addLayout(self.verticalLayout)
        query_panel.setCentralWidget(self.centralwidget)

        self.retranslateUi(query_panel)
        QtCore.QMetaObject.connectSlotsByName(query_panel)

    def retranslateUi(self, query_panel):
        _translate = QtCore.QCoreApplication.translate
        query_panel.setWindowTitle(_translate("query_panel", "查询面板"))
        self.uiPushButton_query.setText(_translate("query_panel", "查询"))
        self.uiRadioButton_pkg2import.setText(_translate("query_panel", "以包名查导入名"))
        self.uiRadioButton_import2pkg.setText(_translate("query_panel", "以导入名查包名"))
        self.uiCheckBox_case_sensitive.setText(_translate("query_panel", "区分大小写"))
        self.label.setText(_translate("query_panel", "查询环境："))
        self.uiLabel_query_environ.setText(_translate("query_panel", "未设置任何环境！"))
        self.uiLabel_query_result.setText(_translate("query_panel", "查询结果："))



# coding: utf-8

from fastpip import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from settings import *
from ui import *
from utils import *

from .messagebox import MessageBox


class IndexUrlManagerWindow(Ui_index_manager, QMainWindow):
    def __init__(self, parent):
        super().__init__(parent)
        self.setupUi(self)
        self.__config = IndexManagerConfig()
        self.__ordered_urls = None
        self.signal_slot_connection()

    def display(self):
        # 首先 resize 为普通大小的窗口不可少，因为：
        # 如果用户关闭最大化状态窗口，我们希望再次打开窗口也是最大化
        # 但是如果再次打开窗口立即以最大化状态显示
        # 那么将缺一个普通大小状态的窗口的尺寸记录
        # 最大化状态窗口的还原按钮将无法把窗口还原为普通大小
        self.resize(*self.__config.window_size)
        if self.isMaximized():
            self.showMaximized()
        else:
            self.showNormal()
        self.__list_widget_urls_update()

    def __store_window_size(self):
        if self.isMaximized() or self.isMinimized():
            return
        self.__config.window_size = self.width(), self.height()

    def closeEvent(self, event: QResizeEvent):
        self.__store_window_size()
        self.__config.save_config()

    @staticmethod
    def __widget_for_list_item(name, url):
        item_layout = QHBoxLayout()
        item_layout.addWidget(QLabel(name))
        item_layout.addWidget(QLabel(url))
        item_layout.setStretch(0, 2)
        item_layout.setStretch(1, 8)
        item_widget = QWidget()
        item_widget.setLayout(item_layout)
        return item_widget

    def __list_widget_urls_update(self):
        self.li_indexurls.clear()
        self.__ordered_urls = tuple(self.__config.index_urls.items())
        for name, url in self.__ordered_urls:
            item_widget = self.__widget_for_list_item(name, url)
            li_item = QListWidgetItem()
            li_item.setSizeHint(QSize(0, 32))
            self.li_indexurls.addItem(li_item)
            self.li_indexurls.setItemWidget(li_item, item_widget)

    def signal_slot_connection(self):
        self.btn_clearle.clicked.connect(self.__clear_line_edit)
        self.btn_saveurl.clicked.connect(self.__save_index_urls)
        self.btn_delurl.clicked.connect(self.__del_index_url)
        self.li_indexurls.clicked.connect(self.__set_url_line_edit)
        self.btn_setindex.clicked.connect(self.__set_global_index_url)
        self.btn_refresh_effective.clicked.connect(self.__display_effective_url)

    def __set_url_line_edit(self):
        selected = self.li_indexurls.currentRow()
        if selected == -1:
            return
        self.le_urlname.setText(self.__ordered_urls[selected][0])
        self.le_indexurl.setText(self.__ordered_urls[selected][1])

    def __clear_line_edit(self):
        self.le_urlname.clear()
        self.le_indexurl.clear()

    def __check_name_url(self, name, url):
        def error(m):
            return MessageBox("错误", m, QMessageBox.Critical)

        if not name:
            msgbox = error("名称不能为空！")
        elif not url:
            msgbox = error("地址不能为空！")
        elif not check_index_url(url):
            msgbox = error("无效的镜像源地址！")
        elif name in self.__config.index_urls:
            msgbox = error(f"名称'{name}'已存在！")
        else:
            return True
        return msgbox.exec_()  # 无论如何都返回 0

    def __save_index_urls(self):
        name = self.le_urlname.text()
        url = self.le_indexurl.text()
        if self.__check_name_url(name, url):
            self.__config.index_urls[name] = url
        self.__list_widget_urls_update()

    def __del_index_url(self):
        selected = self.li_indexurls.currentRow()
        if selected == -1:
            return MessageBox(
                "提示",
                "没有选中列表内的任何条目。",
            ).exec_()
        del self.__config.index_urls[self.__ordered_urls[selected][0]]
        self.__list_widget_urls_update()
        items_count = self.li_indexurls.count()
        if items_count:
            if selected == -1:
                self.li_indexurls.setCurrentRow(0)
            else:
                should_be_selected = (
                    0
                    if selected == 0
                    else selected
                    if selected < items_count
                    else items_count - 1
                )
                self.li_indexurls.setCurrentRow(should_be_selected)

    def __get_cur_environ(self):
        """
        首先使用配置文件中保存的 Python 路径实例化一个 PyEnv，如果路径为空，
        则使用系统环境变量 PATH 中第一个 Python 路径，环境变量中还未找到则返回 None。
        """
        cur_pypaths = self.__config.cur_pypaths
        if not cur_pypaths:
            return PyEnv(cur_py_path())
        for _path in cur_pypaths:
            environ = PyEnv(_path)
            if not environ.env_path:
                continue
            return environ
        else:
            MessageBox(
                "提示",
                "没有找到 Python 环境，请在<包管理器>中添加 Python 目录。",
                QMessageBox.Warning,
            ).exec_()

    def __set_global_index_url(self):
        def warning(m):
            return MessageBox("提示", m, QMessageBox.Warning)

        url = self.le_indexurl.text()
        if not url:
            warn_box = warning("要设置为全局镜像源的地址不能为空！")
        elif not check_index_url(url):
            warn_box = warning("镜像源地址不符合 pip 镜像源地址格式。")
        else:
            environ = self.__get_cur_environ()
            if not environ:
                warn_box = warning(
                    "没找到 Python 环境，全局镜像源设置失败。\n请在<包管理器>中添加 Python 环境。",
                )
            elif environ.set_global_index(url):
                warn_box = MessageBox("提示", f"全局镜像源地址设置成功：\n{url}")
            else:
                warn_box = warning(
                    "全局镜像源设置失败，请确保<包管理器>中第一个环境的 pip 可用或系统环境变量 PATH 中有可用的 Python 路径。"
                )
        warn_box.exec_()

    def __display_effective_url(self):
        environ = self.__get_cur_environ()
        if not environ:
            self.le_effectiveurl.setText("没找到 Python 环境，无法获取镜像源地址。")
            return
        self.le_effectiveurl.setText(
            environ.get_global_index() or "无效的 Python 环境或当前全局镜像源地址为空。"
        )



# coding: utf-8

__doc__ = "检查项目导入的所有模块所需的类、函数等。"

import ast
import re
from os import walk
from os.path import basename, join

from chardet.universaldetector import UniversalDetector
from fastpip import PyEnv


class FindImport(ast.NodeVisitor):
    def __init__(self, out=None):
        if isinstance(out, set):
            self.__result = out
        else:
            self.__result = set()

    def visit_ImportFrom(self, node):
        self.__result.add(self.split(node.module))
        self.generic_visit(node)

    def visit_Import(self, node):
        self.__result.update(self.split(n.name) for n in node.names)
        self.generic_visit(node)

    def visit_Call(self, node):
        if isinstance(node.func, ast.Name):
            if node.func.id == "__import__":
                self.add_call_node_arg(node.args[0])
        elif isinstance(node.func, ast.Attribute):
            attr = node.func.attr
            name = node.func.value
            if isinstance(name, ast.Name) and name.id == "importlib":
                if attr == "__import__" or attr == "import_module":
                    self.add_call_node_arg(node.args[0])
        self.generic_visit(node)

    def getresult(self):
        return self.__result

    @staticmethod
    def split(string: str):
        return string.split(".", 1)[0]

    def add_call_node_arg(self, arg):
        if isinstance(arg, ast.Str):
            self.__result.add(self.split(arg.s))
        elif isinstance(arg, ast.Constant):
            self.__result.add(self.split(arg.value))


def to_be_excluded(_dirpath: str, exclude_dirs):
    _dirpath = _dirpath.lower()
    for p in exclude_dirs:
        if not p:
            continue
        if _dirpath.startswith(p.lower()):
            return True
    return False


def file_codings(project_root, exclude_dirs):
    pattern = re.compile(r"^.+\.py[w]?$")
    path_coding_groups = []
    for root, _, files in walk(project_root):
        if to_be_excluded(root, exclude_dirs):
            continue
        for name in files:
            if not pattern.match(name):
                continue
            path_coding_groups.append(join(root, name))
    coding_detector = UniversalDetector()
    for index, file_path in enumerate(path_coding_groups):
        coding_detector.reset()
        try:
            with open(file_path, "rb") as sf:
                for line in sf:
                    coding_detector.feed(line)
                    if coding_detector.done:
                        break
            coding_detector.close()
            if coding_detector.result["confidence"] < 0.9:
                coding = "utf-8"
            else:
                coding = coding_detector.result["encoding"]
            path_coding_groups[index] = (file_path, coding)
        except Exception:
            path_coding_groups[index] = file_path, None
    return path_coding_groups


class ImportInspector:
    def __init__(self, python_dir, project_root, excludes=None):
        self._root = project_root
        self._excludes = list()
        if isinstance(excludes, (list, tuple)):
            self._excludes.extend(excludes)
        self.importables = self.__project_importables()
        self.importables.update(PyEnv(python_dir).names_for_import())

    @staticmethod
    def __modules_tobe_imported(string):
        """
        查找并返回 string 中所有需要导入的模块集合
        """
        try:
            node = ast.parse(string, "<string>", "exec")
        except:
            return set()
        abstract_syntax_tree_visit = FindImport()
        abstract_syntax_tree_visit.visit(node)
        return abstract_syntax_tree_visit.getresult()

    def __project_importables(self):
        """项目目录下可导入的包、模块。"""
        pkg_pattern = re.compile(r"^[A-Za-z0-9_]+$")
        mod_pattern = re.compile(r"^([A-Za-z0-9_]+).*(?<!_d)\.py[cdw]?$", re.I)
        project_importables = set()
        for root, _, files in walk(self._root):
            if to_be_excluded(root, self._excludes):
                continue
            if "__init__.py" in files:
                project_importables.add(basename(root))
            module_in_dir = False
            for file_name in files:
                matched = mod_pattern.match(file_name)
                if not matched:
                    continue
                module_in_dir = True
                project_importables.add(matched.group(1))
            if module_in_dir:
                pkg_name = pkg_pattern.match(basename(root))
                if pkg_name:
                    project_importables.add(pkg_name.group())
        return project_importables

    def get_missing_items(self):
        """
        查找指定目录内源码所有需要导入的模块，并计算哪些模块未在指定环境中安装
        返回值类型：[(源码文件路径, {源码导入的模块}, {环境中未安装的模块})...]
        """
        results = list()
        for p, c in file_codings(self._root, self._excludes):
            if c is None:
                continue
            try:
                with open(p, encoding=c) as f:
                    file_string = f.read()
                imps = self.__modules_tobe_imported(file_string)
                results.append((p, imps, imps - self.importables))
            except Exception:
                results.append((p, set(), set()))
        if not results:
            results.append((None, set(), set()))
        return results
